#include "Mem.h"
#include "Bit_TDO.h"
#include "Bit_TDI.h"
#include "Bit_TCK.h"
#include "Bit_TMS.h"
#include "Term.h"

#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "self_JTAG.h"
#include "DC18_LCD.h"

void FillDisplay(byte z);

// Most of this JTAG code was taken from the "Flash over JTAG" code in the directory:
// "${PROGRAMFILES}/CodeWarrior for DSC56800E v8.3/ProcessorExpert/Tools/applications/flash_over_jtag/flash_over_jtag_83xx/source/"
// 
// Various changes were made for many reasons, most of can be summarized as us not knowing what we were doing.
// 
// Main change: eonce_* ==> once_*   
// 
// This was because I mistakenly used the 56800 code instead of 56800/E code... easy mistake to make when 
// you have no idea what you are doing.
// 
// =============
//  Weird stuff
// =============
// Much of the following does things which are magical and somewhat odd. Removal of seemingly umimportant lines
// causes failures. Or maybe are just superstitiously linked. Or maybe there is a combination of both.
// 
// There are certainly parts of the code which can be improved by someone more knowledgable about how the
// JTAG state machine, the DSP56800E, and the MC56F8006 all work together. 
// 
// Or by someone who wasn't just ripping someone elses code and trying to make it work.
// 

// =====================
//  Traps for beginners
// =====================
//
// Keep track of the conventions used for the state of each pin at entry to each function. Pay particular
// attention to TMS and TCK.
//
// I didn't initially. 
//
// Sorry. We'll try do better next time. Maybe implement a state machine like I used to keep track
// of the current state now, with a simple: "go to Select-IR-Scan" 
// 
// -- Team Redacted

word data_pl=0; 		/* lengths of JTAG paths */
word instr_pl=0;
word once_base=0xff;
dword once_reply_addr;
dword tapID;
unsigned clearedBits=1;


//#define Term_SendStr(X) (X)
//#define Term_SendNum(X) (X)
//#define Term_SendChar(X) (X)

#define  EXTEST          0x00
#define  SAMPLE          0x01
#define  IDCODE          0x02

#define  ENABLE_ONCE     0x06
#define  DEBUG_REQUEST   0x07
#define  BYPASS          0xffff


#define TMS 0
#define TCK 1
#define TDO 2
#define TDI 3

//#define PROG_FLASH_START 		0x00000000
//#define PROG_FLASH_END   		0x00001fff
//#define PROG_FLASH_SECTOR_SIZE 	0x100


#define JTAG_PATH_LEN_MAX 64 

#define setOrClrVal(reg,mask,val) (reg=(val)?(reg | mask):(reg & ~mask))


#define JTAG_TDI_SET     Bit_TDI_SetVal()
#define JTAG_TDI_RESET   Bit_TDI_ClrVal()

#define JTAG_TMS_SET     Bit_TMS_SetVal() 
#define JTAG_TMS_RESET   Bit_TMS_ClrVal() 

#define JTAG_TCK_SET     Bit_TCK_SetVal(), Cpu_DelayQuick(1), update_state()
#define JTAG_TCK_RESET   Bit_TCK_ClrVal(), Cpu_DelayQuick(1 )

#define JTAG_TDO_VALUE   Bit_TDO_GetVal() 


#define JTAG_TDI_ASSIGN(i) setOrClrVal(GPIO_D_DR,Bit_TDI_PIN_MASK,(byte)((i)&1))
//#define JTAG_TDI_ASSIGN(i) Bit_TDI_PutVal((byte)((i)&1))



#define WAIT_100_NS       Cpu_DelayQuick(1)

word cur_JTAG_state=0xf;



/* algorithm for programming normal program flash */
//		hfm_wait:							/* wait for command to finish */
//			brclr	#0x40,x:(r2+0x13),hfm_wait
//		rx_check:							/* wait for input buffer full */
//			brclr	#0x01,x:(r0-2),rx_check
//			move.w	x:(r0),y0				/* read from Rx buffer */
//			move.w	y0,p:(r3)+
//			move.w	#0x20,x:(r2+0x14)		/* write PGM command */
//			move.w	#0x80,x:(r2+0x13)		/* start the command */
//			bra		hfm_wait				/* loop */
//0x00000019  0x8A460013407D         brclr       #0x40,X:(R2+0x13),*+0
//0x0000001C  0xE700                 nop
//0x0000001D  0xE700                 nop
//0x0000001E  0x8A44FFFE017B         brclr       #1,X:(R0-2),*-2
//0x00000021  0xF514                 move.w      X:(R0),Y0
//0x00000022  0xE700                 nop
//0x00000023  0x8563                 move.w      Y0,P:(R3)+
//0x00000024  0x864600200014         move.w      #32,X:(R2+0x14)
//0x00000027  0x864600800013         move.w      #128,X:(R2+0x13)
//0x0000002A  0xA96E                 bra         *-17
const word pgm_write_single_pflash[]=
{	
	0x8A46, 0x0013, 0x407D, 0xE700, 0xE700, 0x8A44, 0xFFFE, 0x017B,
	0xF514, 0xE700, 0x8563, 0x8646, 0x0020, 0x0014, 0x8646, 0x0080,
	0x0013, 0xA96E
};


const char sTable[][2]=
{
    {  0x2,0x5    }, // 0x00 Exit2-DR
    {  0x3,0x5    }, // 0x01 Exit1-DR
    {  0x2,0x1    }, // 0x02 Shift-DR
    {  0x3,0x0    }, // 0x03 Pause-DR
    {  0xe,0xf    }, // 0x04 Select-IR-Scan
    {  0xc,0x7    }, // 0x05 Update-DR
    {  0x2,0x1    }, // 0x06 Capture-DR
    {  0x6,0x4    }, // 0x07 Select-DR-Scan
    {  0xa,0xd    }, // 0x08 Exit2-IR
    {  0xb,0xd    }, // 0x09 Exit1-IR
    {  0xa,0x9    }, // 0x0a Shift-IR
    {  0xb,0x8    }, // 0x0b Pause-IR
    {  0xc,0x7    }, // 0x0c Run-Test/Idle
    {  0xc,0x7    }, // 0x0d Update-IR
    {  0xa,0x9    }, // 0x0e Capture-IR
    {  0xc,0xf    }  // 0x0f Test-Logic-Reset
};




// About the following code:
//
//  After Table 5 ("56F8006/56F8002 Signal and Package Information")
//  in "MC56F8006.pdf", p. 17
//
//  +---------------------------------------------------------------------+
//  |RESET (GPIOA7) LQFP32 Pin 15                                         |
//  +---------------------------------------------------------------------+
//  |Type: Input/Output                                                   |
//  +---------------------------------------------------------------------+
//  |State During Reset: Input, internal pullup enabled                   |
//  +---------------------------------------------------------------------+
//  |Reset - This input is a direct hardware reset on the processor.      |
//  |When RESET is asserted low, the device is initialized and placed in  |
//  |the reset state. A Schmitt-trigger input is used for noise immunity. |
//  |The internal reset signal is deasserted synchronous with the         |
//  |internal clocks after a fixed number of internal clocks.             |
//  |                                                                     |
//  |Port A GPIO - This GPIO pin can be individually programmed as        |
//  |an input or output pin. RESET functionality is disabled in this mode |
//  |and the chip can be reset only via POR, COP reset, or software       |
//  |reset.                                                               |
//  |                                                                     |
//  |After reset, the default state is RESET.                             |
//  +---------------------------------------------------------------------+
//
//
// Additionally, in Section 6.3 (p. 33):
// 
// ... The external pin reset function is shared with an GPIO port A7 on the 
// RESET/GPIOA7 pin. The reset function is enabled following any reset of the 
// device. Bit 7 of GPIOA_PER register must be cleared to use this pin as an GPIO 
// port pin. When enabled as the RESET pin, an internal pullup device is 
// automatically enabled.
// 


// To send a reset on this pin: 
// void dropRESET()
// {
//  Clear 0x80 on GPIOA_PER
//  Set   0x80 on GPIOA_DDR 
//  Clear 0x80 on GPIOA_DR
//  Wait
// }



/*
 * See Section 8.4.2 of "MC56F8006RM.pdf", specifically 8.4.2.2 - 8.4.2.4
 * 
 * Peripheral Enable Register PER  0 == 'GPIO'   1 == 'Peripheral Mode' 
 * (SIM_GPSA is not used as it's either GPIO or RESET; nothing else)
 *    Data Direction Register DDR  0 == INPUT    1 == OUTPUT
 *    Data Register           DR   0 == LOW      1 == HIGH
 * 
 *
 * For GPIOA7/RESET, default is RESET, default to PER=1, DDR=0, DR=1
 * on entry to function:
 *
 */

static void dropRESET()
{
	Term_SendStr("RST 0");
	Term_CRLF();

	Term_SendNum(GPIO_A_PER&0x80?1:0);
	Term_SendNum(GPIO_A_DDR&0x80?1:0);
	Term_SendNum(GPIO_A_RAWDATA&0x80?1:0);
	Term_SendNum(GPIO_A_DR&0x80?1:0);
	Term_SendNum(GPIO_A_PUR&0x80?1:0);
	Term_CRLF();

	Cpu_DisableInt();
	clrReg16Bits(GPIO_A_PER, 0x80U);
	Cpu_Delay100US(100);
	setReg16Bits(GPIO_A_DDR, 0x80U);
	clrReg16Bits(GPIO_A_PUR, 0x80U);
	clrReg16Bits(GPIO_A_DR, 0x80U);
	Cpu_Delay100US(100);
	Cpu_EnableInt();
 
	Term_SendNum(GPIO_A_PER&0x80?1:0);
	Term_SendNum(GPIO_A_DDR&0x80?1:0);
	Term_SendNum(GPIO_A_RAWDATA&0x80?1:0);
	Term_SendNum(GPIO_A_DR&0x80?1:0);
	Term_SendNum(GPIO_A_PUR&0x80?1:0);

	Term_CRLF();
}


static void giveBackRESET()
{
	
	Term_SendStr("RST=1,gb");
	Term_CRLF();

	Term_SendNum(GPIO_A_PER&0x80?1:0);
	Term_SendNum(GPIO_A_DDR&0x80?1:0);
	Term_SendNum(GPIO_A_RAWDATA&0x80?1:0);
	Term_SendNum(GPIO_A_DR&0x80?1:0);
	Term_SendNum(GPIO_A_PUR&0x80?1:0);
	Term_CRLF();

	// it is safe to be already set, yes?

	Cpu_DisableInt();
	Cpu_Delay100US(100);
	clrReg16Bits(GPIO_A_DDR, 0x80U);
	setReg16Bits(GPIO_A_DR,  0x80U);
	setReg16Bits(GPIO_A_PUR, 0x80U);
	Cpu_Delay100US(1000);
	setReg16Bits(GPIO_A_PER, 0x80U);
	Cpu_EnableInt();
	Term_SendNum(GPIO_A_PER&0x80?1:0);
	Term_SendNum(GPIO_A_DDR&0x80?1:0);
	Term_SendNum(GPIO_A_RAWDATA&0x80?1:0);
	Term_SendNum(GPIO_A_DR&0x80?1:0);
	Term_SendNum(GPIO_A_PUR&0x80?1:0);
	Term_CRLF();
}

void update_state()
{
	int tms=Bit_TMS_GetVal()?1:0;
//	Term_SendChar('j');
//	Term_SendChar('s');
//	Term_SendChar('t');
//	Term_SendChar('=');
//	showData(cur_JTAG_state,4);
//	Term_SendChar('=');
//	showData(Bit_TDI_GetVal(),4);
//	showData(Bit_TDO_GetVal(),4);
//	showData(Bit_TCK_GetVal(),4);
//	showData(Bit_TMS_GetVal(),4);
//	Term_CRLF();
	cur_JTAG_state=sTable[cur_JTAG_state][tms];	
}

void gotoSelectDR()
{
	int i;
	Term_SendChar('s');
	Term_SendChar('e');
	Term_SendChar('l');
	Term_SendChar('e');
	Term_SendChar('c');
	Term_SendChar('t');
	Term_SendChar('d');
	Term_SendChar('r');
	Term_CRLF();
	
	JTAG_TMS_SET;
	// This will put it into a know state. I hope.
	for (i=0;i<10;i++) 
	{
		JTAG_TCK_RESET;	     
		JTAG_TCK_SET;
	}
	JTAG_TMS_RESET;
	JTAG_TCK_RESET;      /* Go to Run-Test-Idle */
	JTAG_TCK_SET;
	JTAG_TMS_SET;        /* Go to Select-DR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	if(cur_JTAG_state!=0x07)
	{
		// bad thing!?	;		// should be in Select-DR
	}
}


//--------------------------------------------------------------------------------------------------------

/* Executes OnCE command */
dword once_instruction_exec(int instruction,int rw,int go,int ex) 
{
	dword ret;
	ret = jtag_data_shift(instruction | (ex<<5)|(go<<6)|(rw<<7),8); 
	return ret;
}

/* writes 8 bits to the JTAG DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
void jtag_data_write8(word data) 
{
	jtag_data_shift(data,8);
}

/* writes 16 bits to the JTAG DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
void jtag_data_write16(word data) 
{
	jtag_data_shift(data,16);
}


/* reads 16 bits from the jtag DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
word jtag_data_read16(void) 
{
	return (unsigned int)jtag_data_shift(0,16);
}

word jtag_data_read8(void) 
{
	return (unsigned int)jtag_data_shift(0,8);
}


/* Transfers 16-bit data into the OnCE through DR-shift of Jtag */
#define once_data_write(data) jtag_data_shift(data,16)

/* Reads 16-bit data from the OnCE through DR-shift of Jtag */
#define once_data_read() jtag_data_shift(0,16)

/* Executes one word DSP instruction */
void once_execute_native_instruction1(word opcode) 
{
	once_instruction_exec(0x04,0,1,0); 
	once_data_write(opcode);
}

/* Executes two word DSP instruction */
void once_execute_native_instruction2(word opcode1, word opcode2)
{
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode1);
	once_instruction_exec(0x04,0,1,0);
	once_data_write(opcode2);
}

/* Executes three word DSP instruction */
void once_execute_native_instruction3(word opcode1, word opcode2,word opcode3)
{
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode1);
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode2);
	once_instruction_exec(0x04,0,1,0);
	once_data_write(opcode3);
}


/* exits debug mode and starts executing code */
#define once_exit_debug_mode()	once_instruction_exec(0x01f,0,0,1)

/* enter debug mode by issuing debug request & enable once jtag commands */
#define once_enter_debug_mode()	(jtag_instruction_exec(0x7,4),jtag_instruction_exec(0x6,4))

/* --------------- Real-time data exchange --------------- */

word once_read_status_reg()
{
	word ret;
	once_instruction_exec(0x03,1,0,0);
	ret= jtag_data_read16();
	return ret; 
}

word once_ctrl_read()
{
	word ret;
	once_instruction_exec(0x01,1,0,0);
	ret=jtag_data_read8();
	return ret;
}

/* writes EOnCE control reg */
void once_ctrl_write(word data)
{
	once_instruction_exec(0x01,1,0,0);
	jtag_data_write8(data);
}

/* writes data into upper ORx register of the target */
void once_tx_upper_data(word data) 
{
	once_instruction_exec(0x0d,0,0,0);
	jtag_data_write16(data);
}

/* reads data from upper OTx register of the target */
word once_rx_upper_data() 
{
	word ret;
	once_instruction_exec(0x09,1,0,0);
	ret=jtag_data_read16();
	return ret;
}

/* reads data from lower OTx register of the target */
#define once_rx_lower_data() (once_instruction_exec(0x07,1,0,0),jtag_data_read16())

/* reads status of OTx & ORx registers */
#define once_data_status() (once_instruction_exec(0x06,1,0,0),jtag_data_read8())

/* reads Eonce base address - XXX this returns only the top 8-bits */
word once_read_base_addr() 
{
	word ret;
	once_instruction_exec(0x05,1,0,0);
	ret = jtag_data_read8();
	return ret;
}

/* measures data and instruction JTAG path lengths */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
/* returns 0 in case measurement has overflown, 1 in case measurement is OK */

word jtag_measure_paths(void) 
{
	word i;
	Term_SendChar('j');
	Term_SendChar('-');
	Term_SendChar('m');
	Term_SendChar('p');
	Term_CRLF();

	SetLEDs(0x01,0);

	JTAG_TMS_SET;								/* Go to Select-IR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TMS_RESET;								/* Go to Capture-IR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-IR */ /* Now the Jtag is in the Shift-IR state */
	JTAG_TDI_ASSIGN(1);
	for (i=0;i<JTAG_PATH_LEN_MAX;i++) 
	{
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
	}
	JTAG_TDI_ASSIGN(0);								/* shift 0 into beginning of the IR path */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TDI_ASSIGN(1);
	i=0;
	SetLEDs(0x03,0);
	while ((JTAG_TDO_VALUE)&&(i<JTAG_PATH_LEN_MAX)) /* wait for the 0 to appear at the path end */

	{ 
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		i++;
	}
	JTAG_TCK_RESET;
	JTAG_TCK_SET;									/* the whole IR path now contains 1 = BYPASS command */
	JTAG_TMS_SET;
	JTAG_TCK_RESET;									/* Go to Exit1-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;									/* Go to Update-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;									/* Go to Select-DR-Scan */
	JTAG_TCK_SET;
	instr_pl=i;
	SetLEDs(0x07,0);
	

	JTAG_TMS_RESET;								/* Go to Capture-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Now the Jtag is in the Shift-DR state */
	JTAG_TDI_ASSIGN(1);
	for (i=0;i<JTAG_PATH_LEN_MAX;i++) 
	{
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
	}
	JTAG_TDI_ASSIGN(0);								/* shift 0 into beginning of the DR path */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TDI_ASSIGN(1);
	i=0;
	SetLEDs(0x0f,0);
	while ((JTAG_TDO_VALUE)&&(i<JTAG_PATH_LEN_MAX)) /* wait for the 0 to appear at the path end */
	{ 
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		i++;
	}
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* the whole DR path now contains 1 */
	JTAG_TMS_SET;
	JTAG_TCK_RESET;								/* Go to Exit1-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Update-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Select-DR-Scan */
	JTAG_TCK_SET;
	data_pl=i;

	SetLEDs(0x1f,0);

	if(
	0 && 
	//(data_pl != 1) ||
	 ((instr_pl != 8) && (instr_pl != 4 )))
	{
		giveBackRESET();
		showBadness(0x20,0x38);
		showData(data_pl, 8);
		showData(instr_pl,8 );
		return 1;
	}
	
	showData(data_pl, 8);
	showData(instr_pl,8 );
	Term_CRLF();
	
	if ((data_pl<JTAG_PATH_LEN_MAX)&&(instr_pl<JTAG_PATH_LEN_MAX))
	{
		return(1); 
	}
	else
	{
		return(0);
	}
}

/* shifts up to 32 bits in and out of the jtag DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
dword jtag_data_shift(dword data, word bit_count) 
{
	int i;
	dword result=0;
	dword curBit=1;

//	Term_SendChar('j');
//	Term_SendChar('-');
//	Term_SendChar('d');
//	Term_SendChar('s');


	JTAG_TMS_RESET;								/* Go to Capture-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Now the Jtag is in the Shift-DR state */

	for (i=0;i<bit_count;i++) 
	{
		JTAG_TDI_ASSIGN(data);
		data>>=1;
		if (i==(bit_count-1)) JTAG_TMS_SET;	/* Go to Exit1-DR */
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		if(JTAG_TDO_VALUE)
		{
			result|=curBit;
		}
		curBit=curBit + curBit;
	}
	
	JTAG_TCK_RESET;								/* Go to Update-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Select-DR-Scan */
	WAIT_100_NS;
	WAIT_100_NS;
	JTAG_TCK_SET;

//	showData(result,0x20);
//	Term_CRLF();
	return(result);
}

/* Executes Jtag command */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
/* the second parameter specifies instruction length in bits */
word jtag_instruction_exec(word instruction, word instr_len) 
{
	word i;
	word status=0;
	dword curBit=1;

//	Term_SendChar('j');
//	Term_SendChar('-');
//	Term_SendChar('i');
//	Term_SendChar('s');

	JTAG_TMS_SET; 								/* Go to Select-IR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TMS_RESET; 								/* Go to Capture-IR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TCK_RESET;
	JTAG_TCK_SET; 								/* Go to Shift-IR */
	/* Now the Jtag is in the Shift-IR state */
	
	for (i=0;i<instr_len;i++) 
	{
		JTAG_TDI_ASSIGN(instruction);
		instruction>>=1;
		if (i==(instr_len-1)) JTAG_TMS_SET; 	/* Go to Exit1-IR */
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		if(JTAG_TDO_VALUE)
		{
			status|=curBit;
		}
		curBit=curBit + curBit;
	}
	JTAG_TCK_RESET; 								/* Go to Update-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET; 								/* Go to Select-DR-Scan */
	JTAG_TCK_SET;

//	showData(status,0x20);
//	Term_CRLF();
	return(status);
}



/* downloads program into target's program RAM */
/* returns: 0 = success, -1 = unable to write to the pram, -2 = unable to access EOnCE */
char once_pgm_dload(const word *pgm, word pgm_size, dword pram_addr) 
{
	int i;
	once_move_long_to_r0(pram_addr);	/* load destination address */
	for (i=0;i<pgm_size;i++) 
	{
		SetLEDs((int)(0x30|(pgm[i]    ))&0x3f,1);
		SetLEDs((int)(0x20|(pgm[i]>> 4))&0x3f,1);
		SetLEDs((int)(0x30|(pgm[i]>> 8))&0x3f,1);
		SetLEDs((int)(0x20|(pgm[i]>>12))&0x3f,1);
		SetLEDs(0x30,0);
		once_move_long_to_r0(pram_addr+i);	/* load destination address */
		once_move_value_to_y0(*(pgm+i));			/* load opcode */
		once_move_y0_at_pr0_inc();					/* store it to PRAM */
	}

	once_move_long_to_r0(pram_addr);	/* load destination address */
	once_move_long_to_r2(((dword)(once_base)<<16)+0xffff);	/* load OTx/ORx reg address */

	for (i=0;i<pgm_size;i++) 
	{
		word tmp;
		once_move_at_pr0_inc_to_y0();				/* load from PRAM */
		once_move_y0_at_r2();
		tmp = once_rx_upper_data();
		if (tmp != *(pgm+i)) 
		{
			showBadness(0x31,0x3);
			showData(*(pgm + i),16);
			showData(tmp,16);
//			print("Downloading of code failed, verify PRAM base address\r\n");
			return (-1);
		}
	}
	return(0);
}



char once_infect_other_badge() 
{
	dword data_offset;

	FillDisplay(0x13);
	Display(0,N_ROWS);

	if(once_pgm_dload(pgm_write_single_pflash, sizeof(pgm_write_single_pflash)/sizeof(word),pram_exec_addr))
	{
		Term_SendStr("failed pgm dload");
		Term_CRLF();
		return -1;
	}
	
	Term_SendStr("pgm dld.");
	once_move_long_to_r2(flash_base);						/* load HFM base to r2 */
	once_move_long_to_r0(once_reply_addr );					/* load OTx/ORx reg address to r0 */

	once_move_value_at_r2_disp(0x00,0x01);			 		/* write to HFM_CR (lock=0, select bank) */
	once_move_value_at_r2_disp(0x04,0x13);					/* write to HMF_USTAT, clear PVIOL, ACCERR & BLANK bits */
	once_move_value_at_r2_disp(0x10,0x13);					/* clear only one bit at a time */
	once_move_value_at_r2_disp(0x20,0x13);
	once_move_value_at_r2_disp(0x00,0x10);					/* write to HMF_PROT, clear protection */
	once_move_value_at_r2_disp(0x00,0x11);					/* write to HMF_PROTB, clear protection */

	data_offset=0;

	once_move_long_to_r3(PROG_FLASH_START);							/* start of flash */
	once_move_at_r0_y0();											/* clear the ORx buffer */
	once_move_value_to_pc(pram_exec_addr);								/* load PC */	
	once_exit_debug_mode();											/* run the program */
	Term_SendStr("doing flash...");
	while (data_offset + PROG_FLASH_START<=PROG_FLASH_END)						/* loop through all the data */
	{		   	
		word curWord;
		word ods;
		word q;
		int r=0;
		int abc=(int)data_offset >> 6;
		ods=once_data_status();
		q=0;
		
		while (ods&0x01)
		{
			SetLEDs(0x30 | ((int)(ods&0xf)) ,0);
			Cpu_Delay100US(500);
			ods=once_data_status();
			if((ods&1) ==0)
			{
				r=0;
				q=0;
				break;	
			}

			SetLEDs(0x00 ,1);
			if(++q < 0x8000)
			{
				q=0;

				SetLEDs(r ,100);
				if(++r>0x40)
				{
					showBadness(0x10,0x02);
					showData(data_offset,16);
					Term_SendStr("flash failed lots");
					Term_CRLF();
					return -1;	
				}
			}
			
		}
		
		curWord=pmem_read((word*)data_offset);

		curWord=check_for_patch(data_offset,curWord);
		
		once_tx_upper_data(curWord);								/* now flash the word */
		data_offset++;												/* increment the pointer */


		SetLEDs( abc ,0);
		if(data_offset&0x20)
		{
			abc |= 0x20;
		}
		else
		{
			abc |=0x30;
		}
		SetLEDs( abc ,0);
	}
	once_enter_debug_mode();										/* back to debug mode */
	Term_SendStr("finished flash...");
	Term_CRLF();
	FillDisplay(0x88);
	Display(0,N_ROWS);
	return(0);
}

/* programs clock divider */
char once_set_hfmdiv() 
{
	word i;
	once_move_long_to_r2(flash_base);						/* load HFM base to r2 */
	once_move_long_to_r0(((dword)(once_base)<<16)+0xffff);	/* load OTx/ORx reg address */
	once_move_at_r2_to_y0();											/* read HFM_CLKD */
	once_move_y0_at_r0();
	i=once_rx_upper_data();
	if ((i&0x80)&&(i&0x7f!=flash_clkdiv)) 
	{
		showBadness(0x31,0x7);
//		print("HFM CLK divisor already set and contains incorrect value (0x%02x)\r\n",i&0x7f);
		return(-1);
	}
	once_move_value_at_r2(flash_clkdiv);					/* write HFM_CLKD */
	once_move_at_r2_to_y0();											/* verify HFM_CLKD */
	once_move_y0_at_r0();
	i=once_rx_upper_data();
	if (i!=(0x80|(flash_clkdiv&0x7f))) 
	{
		showBadness(0x31,0xf);
		Term_SendStr("Unable to set HFM CLK divisor");
		Term_CRLF();
		return(-2);
	}
//	print("HFM CLK divisor set to 0x%02x\r\n",i&0x7f);
	return(0);
}

/* erases flash */
/* returns 0 - OK, !=0 - erase failed */
char once_flash_erase()
{
	word i;
	dword address=PROG_FLASH_START;	/* address inside the page */

	once_move_long_to_r2(flash_base);		/* load HFM base to r2 */
	once_move_long_to_r0(once_reply_addr );	/* load OTx/ORx reg address */

	i=once_hfm_execute_command(0x05,address); /* blank check */

	if (i&0x30) 
	{
//		print("Unexpected EraseVerify HFM command execution error\r\n");
		showBadness(0x3c,0x31);	
		return(-1);
	}


	address=PROG_FLASH_START;						/* start of block */
	while (address<=(PROG_FLASH_END-0x10))							/* until end of flash unit */
	{													/* or until end of this block */
		i=once_hfm_execute_command(0x40,address);	/* erase page */
		if (i&0x30) 
		{
			showBadness(0x3c,0x32);	
//			print("Unexpected PageErase HFM command execution error\r\n");
			return(-1);
		}
		address+=PROG_FLASH_SECTOR_SIZE;
	}
	return(0);
}


/* expects that R2 holds HFM base, R0 holds OTx/ORx register address */
/* returns HMF_USTAT after executing the command */


word once_hfm_execute_command(byte command, dword address) 
{
	word i;
	do 
	{
		once_move_at_r2_disp_to_y0(0x13);		/* read HMF_USTAT */
		once_move_y0_at_r0();
		i=once_rx_upper_data();
	} while (!(i&0x40));						/* wait until current command is complete */

	once_move_value_at_r2_disp(0,0x01);  		/* write to HFM_CR (lock=0, select bank) */
	once_move_value_at_r2_disp(0x04,0x13);		/* write to HMF_USTAT, clear PVIOL, ACCERR & BLANK bits */
	once_move_value_at_r2_disp(0x10,0x13);		/* clear only one bit at a time */
	once_move_value_at_r2_disp(0x20,0x13);
	once_move_value_at_r2_disp(0x00,0x10);		/* write to HMF_PROT, clear protection */
	once_move_value_at_r2_disp(0x00,0x11);		/* write to HMF_PROTB, clear protection */
	once_move_long_to_r3(address);				/* write to the flash block */
	once_move_y0_at_pr3_inc();
	once_move_value_at_r2_disp(command,0x14);	/* write command to the HFM_CMD reg */
	once_move_value_at_r2_disp(0x80,0x13);		/* start the command */
	do 
	{
		once_move_at_r2_disp_to_y0(0x13);		/* read HMF_USTAT */
		once_move_y0_at_r0();
		i=once_rx_upper_data();
	} 
	while (!(i&0x40));							/* wait until the command is complete */
	return(i);
}



int doIDCODE(dword id,dword mask,int iLen)
{	
	word cmdRes;
	if(instr_pl != iLen)
	{
		systemRunState= 1;
		return 0;	
	}

	Term_SendStr("doIDCODE");
	Term_CRLF();
	// 0x02211004
	// 0x01f2601D (seems to be 0x01c0601D ?)
	cmdRes=jtag_instruction_exec(IDCODE,instr_pl);

	tapID = jtag_data_shift(0x00000000,32);

	showData(tapID,32);
	if((tapID & mask) != id)
	{
		if(tapID == 0) 
		{
			systemRunState=3;
		}
		else
		{
			showData(tapID,32);
			systemRunState=4;
		}
		return 0;
	}
	return 1;
}

static int reallyDoInitTap()
{
	word cmdRes;
	word i;

	dropRESET();
	
	gotoSelectDR();

	Term_SendChar('D');
	Term_SendChar('R');
	Term_CRLF();

	if(!jtag_measure_paths())
	{
		systemRunState=4;
		return -1;
	}

	if(instr_pl != 4)
	{
		if(instr_pl != 8)
		{
			Term_SendChar('b');
			Term_SendChar('a');
			Term_SendChar('d');
			Term_CRLF();
			systemRunState=5;	
			return -1;
		}

		if(select_core_tap() == 0)
		{
			systemRunState=6;
			if(instr_pl != 4)
			{
				systemRunState|=0x10;
			}
			
			return -1;
		}


		cmdRes = jtag_instruction_exec(DEBUG_REQUEST,4);

		giveBackRESET();

		cmdRes = jtag_instruction_exec(ENABLE_ONCE,4);            

		
		if(select_master_tap() == 0)
		{
			systemRunState=7;
			return -1;
		}
		
		if(!doIDCODE(0x601D,0xffff,8))
		{
			// correct systemRunState set already; no need add bit for 'in core'
			return -1;	
		}
	}

	if(select_core_tap() == 0)
	{
		systemRunState=8;
		return -1;
	}

	if( instr_pl != 4 )
	{
		systemRunState=9;
		return -1;
	}

	if(!doIDCODE(0x02211004,0xffffffff,4))
	{
		systemRunState|=0x10;
		// systemRunState is already set 
		return -1;	
	}

	
	for(i = 0; i<15; i++)
	{
		cmdRes = jtag_instruction_exec(DEBUG_REQUEST,4);
		if(cmdRes == 0x0d)break;
	}

	if(cmdRes != 0x0d)
	{
		showBadness(0x18 ,(int)cmdRes|0x30);
		systemRunState=10;
		return -1;
	}
	
	giveBackRESET();
	
	for(i = 0; i<5; i++)
	{
		cmdRes = jtag_instruction_exec(ENABLE_ONCE,4);            
		if(cmdRes == 0x0d)break;
	}

	if(cmdRes != 0x0d)
	{
		showBadness(0x18,(int)cmdRes|0x30);
		systemRunState=11;
		return -1;
	}

	
	once_base = once_read_base_addr();
	once_reply_addr = ((dword)(once_base)<<16)+0xffff;
	once_move_long_to_r0(once_reply_addr);
	once_move_omr_to_y0();			/* read OMR */
	once_move_y0_at_r0();    
	i = once_rx_upper_data();

	if ((i&0x01) == 1) 
	{
		systemRunState=12; // XXX is this ever going to be a problem?
		return -1;
	}
	return 0;
}

// wrapper around the actual tap initialization code to catch the RESET line
int doInitTap()
{
	int res;
	Term_SendStr("dIT called");
	res=reallyDoInitTap();
	giveBackRESET();
	Term_SendStr("dIT returned \0");
	Term_SendNum(res);
	Term_CRLF();
	return res;
}


void jtag_disconnect(void) 
{
	Term_SendStr("j-disc");
	if(instr_pl == 4)
	{
		once_move_value_to_pc((dword)0);// reset vector
		once_exit_debug_mode();
		jtag_instruction_exec(0x2,4);
		instr_pl = 0;
	}
	dropRESET();
	Cpu_Delay100US(2000);
	giveBackRESET();
	Cpu_Delay100US(2000);
	Term_CRLF();
} 

void clearBits()
{
	jtag_disconnect();
	Bit_TCK_SetVal();
	Bit_TMS_SetVal();
	Bit_TDI_ClrVal();
	clearedBits=1;
}

//--------------------------------------------------------------------------------------------------------

/* selects the CORE TAP Controller by modifying the TLM register */
/* to be used when Master TAP is selected */
dword select_core_tap(void) 
{
	word cmdRes;
	dword dataRes;

	Term_SendStr("sel core.");
	if(instr_pl != 8)
	{
		showData(instr_pl | 0x20,8);
		return 0;
	}

	cmdRes  = jtag_instruction_exec(0x05,8);	/* select TLM */
	dataRes = jtag_data_shift(0x2,4);		/* select core TAP */

	jtag_measure_paths();					/* rescan jtag chain (Core TAP and Master TAP have different intruction lengths) */

	if(instr_pl != 4 ) // || dataRes != 2)
	{
//		showData((instr_pl << 8) | dataRes,8);
		return 0;			
	}
	return 2;
}


/* selects the Master TAP Controller by modifying the TLM register */
/* to be used when Core TAP is selected */
dword select_master_tap(void) 
{
	word cmdRes;
	dword dataRes;

	Term_SendStr("sel mstr.");
	SetLEDs(0x11,5000);
	SetLEDs(0x22,5000);
	SetLEDs(0x04,5000);
	SetLEDs(0x28,5000);
	SetLEDs(0x18,5000);

	if(instr_pl != 4)
	{
		showData(instr_pl | 0x80,8);
		return 0;
	}
	cmdRes  = jtag_instruction_exec(0x08,4);	/* select TLM */
	dataRes = jtag_data_shift( 0x1,4);			/* select master TAP */
	jtag_measure_paths();										/* rescan jtag chain (Core TAP and Master TAP have different intruction lengths) */
	if(instr_pl != 8 ) // || dataRes != 1)
	{
//		showData((instr_pl <<8)| dataRes,8);
		return 0;			
	}
	return 1;
}

 