/** ###################################################################
**     THIS COMPONENT MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename  : Mem.C
**     Project   : self_JTAG
**     Processor : MC56F8006_32_LQFP
**     Component : IntFLASH
**     Version   : Component 02.362, Driver 02.07, CPU db: 3.00.188
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/27/2010, 5:06 PM
**     Abstract  :
**         This bean "IntFLASH" implements an access to internal FLASH.
**         The bean support reading/writing data into FLASH, erasing of
**         selected sector.
**         The bean supports events if the write interrupt is supported.
**         The bean supports following modes of write operations:
**           - Write - writing without any test.
**           - Destructive write - sector is erased if necessary.
**           - Safe write - user event is invoked to save and resore data
**                          from the current sector.
**         The bean requires on-chip FLASH memory (not used/allocated by
**         other beans).
**     Settings  :
**         FLASH memory type                 : Program FLASH
**         Memory size                       : 8192 words
**         Sector size                       : 256 words
**         Interrupt service                 : Disabled
**         Write method                      : Destructive write (with erase)
**         Wait in RAM                       : yes
**     Contents  :
**         Busy         - bool Mem_Busy(byte Block);
**         SetWordFlash - byte Mem_SetWordFlash(Mem_TAddress Addr, word Data);
**         DataPtr2Addr - Mem_TAddress Mem_DataPtr2Addr(void* Addr);
**         FuncPtr2Addr - Mem_TAddress Mem_FuncPtr2Addr(void(*Addr)());
**
**     Copyright : 1997 - 2009 Freescale Semiconductor, Inc. All Rights Reserved.
**     
**     http      : www.freescale.com
**     mail      : support@freescale.com
** ###################################################################*/
/* MODULE Mem. */

#include "Mem.h"

/* Registers for usage in asm macros/functions */
#define FMCMD_ASM X:62484
#define FMCR_ASM X:62465
#define FMUSTAT_ASM X:62483

static byte Err;                       /* Error state of current process */

/*
** ===================================================================
**     Method      :  Mem_OutOfRange (component IntFLASH)
**
**     Description :
**         The method returns FALSE, if both of the input parameters are 
**         in an allowed flash memory address range. If at least one of 
**         the parameters is out of the range, return TRUE.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static byte OutOfRange(Mem_TAddress addr1,Mem_TAddress addr2)
{
  return ((addr1>addr2)||(addr1<PROG_FLASH_START)||(addr2>PROG_FLASH_END));
}

/*
** ===================================================================
**     Method      :  Mem_SectorSize (component IntFLASH)
**
**     Description :
**         The method returns flash memory sector (erase page) size of a 
**         desired area.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define SectorSize(addr) (PROG_FLASH_SECTOR_SIZE)

/*
** ===================================================================
**     Method      :  ClearFlags (component IntFLASH)
**
**     Description :
**         The method clears the flash memory interface complete and 
**         error flags.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define ClearFlags() setReg(FM_USTAT,0x30)

/*
** ===================================================================
**     Method      :  readflash (component IntFLASH)
**
**     Description :
**         The method reads 16-bit word from a flash memory.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
/*lint -save -e745 -e939 -e533 -e82 Disable MISRA rule (75,83,84) checking. */
static asm word readflash(Mem_TAddress address)
{
  move.l A10,R2;                       /* Move given address to pointer register */
  move.w p:(r2)+,y0;                   /* Read data from program memory */
  rts;
}
/*lint -restore */

/*
** ===================================================================
**     Method      :  writeflash (component IntFLASH)
**
**     Description :
**         The method writes 16-bit word to a flash memory.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
/*lint -save -e745 -e939 -e533 -e82 Disable MISRA rule (75,83,84) checking. */
static asm word writeflash(Mem_TAddress address, word data)
{
  move.l A10,R2;                       /* Move given address to pointer register */
  move.w Y0,p:(R2)+;                   /* Write given data to desired address in program memory */
  rts;
}
/*lint -restore */

/*lint -save -e???? Disable MISRA rule (all) checking. */
/*
** ===================================================================
**     Method      :  Run_and_wait_in_prog_flash (component IntFLASH)
**
**     Description :
**         This method runs programming or erasing process end waits 
**         until it ends. This routine is placed in program flash memory.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#pragma define_section RAW_PROG_FLASH_section ".RAW_PROG_FLASH" RX
#pragma section RAW_PROG_FLASH_section begin
static asm void Run_and_wait_in_prog_flash(Mem_TAddress address, word data, word command)
{
  move.l A10,R2;                       /* Move given address to pointer register */
  move.w Y0,p:(R2)+;                   /* Write data to the address in program memory */
  move.w Y1,FMCMD_ASM;
  move.w #$80,FMUSTAT_ASM;
wait:
  moveu.w #$f020,R0;
  adda #$3f3,R0,R0;
  move.w X:(R0),A;
  bftstl #$40,A1;
  bcs wait;
  rts;
}
#pragma section RAW_PROG_FLASH_section end
/*lint -restore */

/*lint -save -e???? Disable MISRA rule (all) checking. */
/*
** ===================================================================
**     Method      :  Run_and_wait_in_RAM (component IntFLASH)
**
**     Description :
**         This method runs programming or erasing process end waits 
**         until it ends. This routine is placed in RAM.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#pragma define_section RAW_RAM_section ".RAW_RAM" RWX
#pragma section RAW_RAM_section begin
static asm void Run_and_wait_in_RAM(Mem_TAddress address, word data, word command)
{
  move.l A10,R2;                       /* Move given address to pointer register */
  move.w Y0,p:(R2)+;                   /* Write data to the address in program memory */
  move.w Y1,FMCMD_ASM;
  move.w #$80,FMUSTAT_ASM;
wait:
  moveu.w #$f020,R0;
  adda #$3f3,R0,R0;
  move.w X:(R0),A;
  bftstl #$40,A1;
  bcs wait;
  rts;
}
#pragma section RAW_RAM_section end
/*lint -restore */

/*lint -save -e???? Disable MISRA rule (all) checking. */
/*
** ===================================================================
**     Method      :  mempcpy (component IntFLASH)
**
**     Description :
**         The methods makes a copy of a program memory block (both 
**         source and destination address is in a program memory). It is 
**         used for copying the WIR_routine / Run_and_wait_in_RAM routine 
**         to RAM after system reset.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static asm void mempcpy(Mem_TAddress source_addr, Mem_TAddress dest_addr, word size)
{
    move.w  y0,r2                      /* set data size */
    move.l  a10,r3                     /* src address */
    move.l  b10,r1                     /* dest address */
    do      r2,>>end_prom2pram         /* copy for r2 times */
    move.w  p:(r3)+,x0                 /* fetch value at address r3 */
    nop
    nop
    nop
    move.w  x0,p:(r1)+                 /* stash value at address r1 */
    nop
    nop
    nop
end_prom2pram:
    rts;
}
/*lint -restore */

/*
** ===================================================================
**     Method      :  procflash (component IntFLASH)
**
**     Description :
**         The method programs 16-bit word to a flash memory.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static byte procflash(Mem_TAddress address, word data, word command)
{
  if (Mem_Busy())                      /* Is a command in progress in any of the flash devices? */
    return ERR_BUSY;                   /* If yes then error */
  ClearFlags();                        /* Clear all flags */
  EnterCritical();                     /* Disable all low level interrupts */
  Run_and_wait_in_RAM(address,data,command); /* Process routine in RAM */
  ExitCritical();                      /* Enable all low level interrupts */
  if ((getRegBit(FM_USTAT,PVIOL))||(getRegBit(FM_USTAT,ACCERR))) /* Is protection violation or access error detected ? */
    return ERR_NOTAVAIL;               /* If yes then return the error */
  return Err;
}

/*
** ===================================================================
**     Method      :  Mem_Busy (component IntFLASH)
**
**     Description :
**         Method return status of the FLASH device
**     Parameters  :
**         NAME            - DESCRIPTION
**         Block           - Block number. Not used for Freescale
**                           56800 and HCS12X derivates - any value may be passed.
**     Returns     :
**         ---             - TRUE/FALSE - FLASH is busy/ready
** ===================================================================
*/
/*
bool Mem_Busy(byte Block)

**      This method is implemented as macro. See Mem.h file.      **
*/

/*
** ===================================================================
**     Method      :  Mem_SetWordFlash (component IntFLASH)
**
**     Description :
**         Write word to address in FLASH.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Addr            - Address to FLASH.
**         Data            - Data to write.
**     Returns     :
**         ---             - Error code, possible codes:
**                           - ERR_OK - OK
**                           - ERR_NOTAVAIL - Desired program/erase
**                           operation is not available
**                           - ERR_RANGE - Address is out of range
**                           - ERR_VALUE - Read value is not equal to
**                           written value
**                           - ERR_SPEED - This device does not work
**                           in the active speed mode
** ===================================================================
*/
byte Mem_SetWordFlash(Mem_TAddress Addr,word Data)
{
  Err = ERR_OK;
  if (OutOfRange(Addr,Addr))            /* Check range of address */
    return(ERR_RANGE);
  if (~(readflash(Addr))&Data) {       /* Is a sector erasure necessary? */
    Err = procflash(Addr,0,PAGE_ERASE); /* Erase sector */
    if (Err!=ERR_OK)  {                /* If an error occured then exit */
      return Err;
    }
    while (getRegBit(FM_USTAT,CBEIF) == 0) {} /* Wait to buffer empty */
  }
  Err = procflash(Addr, Data, PROGRAM); /* Write new data to Flash */
  while (getRegBit(FM_USTAT,CCIF) == 0) {} /* Wait to command complete */
  if (Err==0) {
    if (readflash(Addr) != Data) {     /* If the read data are not equal written data then error */
      return ERR_VALUE;
    }
  }
  return Err;
}

/*
** ===================================================================
**     Method      :  Mem_Init (component IntFLASH)
**
**     Description :
**         Initializes the associated peripheral(s) and the beans 
**         internal variables. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void Mem_Init(void)
{
/* Copy FLASH process routine to program RAM. All program or erase processes will be launched and wait for their end in this routine */
  mempcpy(((dword)&_LWIR_ROM_addr_high<<16)+(dword)&_LWIR_ROM_addr_low, ((dword)&_LWIR_RAM_addr_high<<16)+(dword)&_LWIR_RAM_addr_low, (word)(dword)&_LWIR_size);
}

/*
** ===================================================================
**     Method      :  Mem_DataPtr2Addr (component IntFLASH)
**
**     Description :
**         This method converts data pointer to format of a bean method
**         address parameter. Generally a data pointer format is
**         different from format of a method Addr parameter.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Addr            - Data pointer.
**     Returns     :
**         ---             - Address in the format used in the bean
**                           methods.
**                           Version specific information for 56800
**                           derivatives ] 
**                           The return value is address of a byte
**                           location of the object the input parameter
**                           - pointer pointing at (address of a 16 bit
**                           word location multiplied by 2 and the least
**                           significant bit determines even or odd byte).
** ===================================================================
*/
/*
Mem_TAddress Mem_DataPtr2Addr(void DataPtr)

**  This method is implemented as a macro. See Mem.h file.  **
*/

/*
** ===================================================================
**     Method      :  Mem_FuncPtr2Addr (component IntFLASH)
**
**     Description :
**         This method converts function pointer to format of a bean
**         method address parameter. Generally a function pointer
**         format is different from format of a method Addr parameter.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Addr            - Function pointer.
**     Returns     :
**         ---             - Address in the format used in the bean
**                           methods.
**                           Version specific information for 56800
**                           derivatives ] 
**                           The return value is address of a byte
**                           location of the object the input parameter
**                           - pointer pointing at (address of a 16 bit
**                           word location multiplied by 2 and the least
**                           significant bit determines even or odd byte).
** ===================================================================
*/
/*
Mem_TAddress Mem_FuncPtr2Addr(void FuncPtr)

**  This method is implemented as a macro. See Mem.h file.  **
*/

/* END Mem. */
/*
** ###################################################################
**
**     This file was created by Processor Expert 3.00 [04.35]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
