/** ###################################################################
**     Filename  : self_JTAG.C
**     Project   : self_JTAG
**     Processor : MC56F8006_48_LQFP
**     Version   : Driver 01.14
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/5/2010, 1:49 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
** ###################################################################*/
/* MODULE self_JTAG */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "Bit_TDO.h"
#include "Bit_TDI.h"
#include "Bit_TCK.h"
#include "Bit_TMS.h"
#include "Button.h"
#include "Inhr1.h"
#include "LED1.h"
#include "LED2.h"
#include "LED3.h"
#include "LED4.h"
#include "LED5.h"
#include "LED6.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "Bit_TDI.h"
#include "Bit_TCK.h"
#include "Bit_TMS.h"
#include "self_JTAG.h"

unsigned clearedBits=1;
unsigned long tapID;

int isReading=0;
unsigned int data_pl=0; 		/* lengths of JTAG paths */
unsigned int instr_pl=0;
unsigned int data_pp=0; 		/* position of the part in the JTAG chain, 0=beginning */
unsigned int instr_pp=0;
unsigned int once_base;
unsigned long once_reply_addr;

int intFlag;

char HexChars[]="0123456789abcdef";
char RESTART[]="Restart";
char ENDOFOUTPUT[]="Ends";
int sysRunState= 0;

asm void Cpu_DelayQuick(word us10)
{
  /* Total irremovable overhead: about 16 cycles */
  /* move.w: 2 cycles overhead (load parameter into register) */
  /* jsr:    5 cycles overhead (jump to subroutine) */
  /* rts:    8 cycles overhead (return from subroutine) */
  /* nop:    1 cycles overhead (aditional nops) */

  loop:
  /* 10 us delay block begin */
  /*
   * Delay
   *   - requested                  : 100 us @ 32MHz,
   *   - possible                   : 3200 c, 100000 ns
   *   - without removable overhead : 319 c, 9981.2499999999987 ns
   */
  adda #2, SP                          /* (1 c: 31.25 ns) move SP forward */
  move.l A10, X:(SP)                   /* (2 c: 62.5 ns) push A */
  move.w #317, A                       /* (2 c: 62.5 ns) number of iterations */
  do A, label0                         /* (8 c: 250 ns) repeat 3178x nop */
	nop                                /* (1 c: 31.25 ns) wait for 1 c */
  label0:
  move.l X:(SP), A                     /* (2 c: 62.5 ns) pop A */
  suba #2, SP                          /* (1 c: 31.25 ns) move SP back */
  /* 10 us delay block end */
  dec.w Y0                             /* us10 parameter is passed via Y0 register */
  jne loop                             /* next loop */
  nop                                  /* avoid pipeline conflicts */
  rts                                  /* return from subroutine */
}



#define  EXTEST          0x00
#define  SAMPLE          0x01
#define  IDCODE          0x02

#define  ENABLE_ONCE     0x06
#define  DEBUG_REQUEST   0x07
#define  BYPASS          0xffff


#define TMS 0
#define TCK 1
#define TDO 2
#define TDI 3

#define JTAG_PATH_LEN_MAX 64 

#define JTAG_TDI_SET     Bit_TDI_SetVal() ,Cpu_DelayQuick(250)
#define JTAG_TDI_RESET   Bit_TDI_ClrVal() ,Cpu_DelayQuick(250)

#define JTAG_TMS_SET     Bit_TMS_SetVal(), Cpu_DelayQuick(250)
#define JTAG_TMS_RESET   Bit_TMS_ClrVal(), Cpu_DelayQuick(250)  

#define JTAG_TCK_SET     Bit_TCK_SetVal() ,Cpu_DelayQuick(1000),update_state()
#define JTAG_TCK_RESET   Bit_TCK_ClrVal() ,Cpu_DelayQuick(1000)

#define JTAG_TDO_VALUE   Bit_TDO_GetVal() 

#define JTAG_TDI_ASSIGN(i) Bit_TDI_PutVal((unsigned char)((i)&1))


#define WAIT_100_NS       Cpu_DelayQuick(250)

unsigned int curState=0xf;

void update_state();

void SetLEDs(int val,unsigned int t);
void SetLED3(int r,int y, int g,unsigned int t);
void getPush(int p,int q);

const char sTable[0x10][2]=
{
    {  0x2,0x5    }, // 0x00 Exit2-DR
    {  0x3,0x5    }, // 0x01 Exit1-DR
    {  0x2,0x1    }, // 0x02 Shift-DR
    {  0x3,0x0    }, // 0x03 Pause-DR
    {  0xe,0xf    }, // 0x04 Select-IR-Scan
    {  0xc,0x7    }, // 0x05 Update-DR
    {  0x2,0x1    }, // 0x06 Capture-DR
    {  0x6,0x4    }, // 0x07 Select-DR-Scan
    {  0xa,0xd    }, // 0x08 Exit2-IR
    {  0xb,0xd    }, // 0x09 Exit1-IR
    {  0xa,0x9    }, // 0x0a Shift-IR
    {  0xb,0x8    }, // 0x0b Pause-IR
    {  0xc,0x7    }, // 0x0c Run-Test/Idle
    {  0xc,0x7    }, // 0x0d Update-IR
    {  0xa,0x9    }, // 0x0e Capture-IR
    {  0xc,0xf    }  // 0x0f Test-Logic-Reset
};

void update_state()
{
	int tms=Bit_TMS_GetVal()?1:0;
	curState=sTable[curState][tms];	
}

void SetLEDs(int val,word t)
{
	LED1_PutVal((unsigned char)(val&0x01));
	LED2_PutVal((unsigned char)(val&0x02));
	LED3_PutVal((unsigned char)(val&0x04));
	LED4_PutVal((unsigned char)(val&0x08));
	LED5_PutVal((unsigned char)(val&0x10));
	LED6_PutVal((unsigned char)(val&0x20));
	if(intFlag)return;
	if(t==0) return;
	Cpu_Delay100US(t);	
}

void SetLED3(int r,int y, int g,word t)
{
	SetLEDs(r | (y<<1) | ( g<<2) , t);
}

void getPush(int p,int q)
{
	intFlag=0;
	if(sysRunState != 1)return;
	while(!intFlag)
	{
		SetLEDs(p,1500);
		SetLEDs(q,1500);						
	}
	intFlag=0;
	SetLEDs(0,0);
	Cpu_Delay100US(5000);
}

void showData(unsigned long data,unsigned int bit_length)
{
	int i;
	int bl;

	getPush(0x15,0x2a);
	SetLEDs(0x30,5000);

	bl=(int)bit_length;
	for(i=0;i< bl ;i+=4)
	{
		SetLEDs(0x3f,2000);	
		SetLEDs(0x30,5000);	
	}

	getPush(0x23,0x26);

	for(i=bl-4; i>=0;i-=4)
	{
		int p,q;
		
		p=(int)( (data>>i) & 0xf );
		q=p|0x30;
		if(sysRunState==1 )
		{
			getPush(p,q);
		}
		else
		{
			SetLEDs((int)(q),3500);
			SetLEDs(p,500);			
		}		
	}

	SetLEDs(0x00,0);
	if(sysRunState!=1 )return;

	for(i=0;i<6;i++)
	{
		SetLEDs((1<<i),500);
		SetLEDs((1<<i) | (0x20>>i),100);
		SetLEDs((0x20>>i),200);
	}
}

/* Executes OnCE command */
unsigned long once_instruction_exec(int instruction,int rw,int go,int ex) 
{
	unsigned long ret;
	ret = jtag_data_shift(instruction | (ex<<5)|(go<<6)|(rw<<7),8); 
	return ret;
}


/* writes 8 bits to the JTAG DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
void jtag_data_write8(unsigned int data) 
{
	jtag_data_shift(data,8);
}



/* writes 16 bits to the JTAG DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
void jtag_data_write16(unsigned int data) 
{
	jtag_data_shift(data,16);
}


/* reads 16 bits from the jtag DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
unsigned int jtag_data_read16(void) 
{
	return (unsigned int)jtag_data_shift(0,16);
}

unsigned int jtag_data_read8(void) 
{
	return (unsigned int)jtag_data_shift(0,8);
}


/* Transfers 16-bit data into the OnCE through DR-shift of Jtag */
#define once_data_write(data) jtag_data_shift(data,16)

/* Reads 16-bit data from the OnCE through DR-shift of Jtag */
#define once_data_read() jtag_data_shift(0,16)

/* Executes one word DSP instruction */
void once_execute_instruction1(unsigned int opcode) 
{
	once_instruction_exec(0x04,0,1,0); 
	once_data_write(opcode);
}


/* Executes two word DSP instruction */
void once_execute_instruction2(unsigned int opcode1, unsigned int opcode2)
{
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode1);
	once_instruction_exec(0x04,0,1,0);
	once_data_write(opcode2);
}

/* Executes three word DSP instruction */
void once_execute_instruction3(unsigned int opcode1, unsigned int opcode2,unsigned int opcode3)
{
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode1);
	once_instruction_exec(0x04,0,0,0);
	once_data_write(opcode2);
	once_instruction_exec(0x04,0,1,0);
	once_data_write(opcode3);
}


void once_stop_debugging_and_reset() 
{
	once_instruction_exec(0x04,0,0,0); 
	once_data_write(0xE71E),
	once_instruction_exec(0x04,0,0,0); 
	once_data_write(0);
	once_instruction_exec(0x04,0,1,0); 
	once_data_write(0);    
	once_instruction_exec(0x04,0,1,1); 
	once_data_write(0xe700);
}


unsigned int once_status_read()
{
	unsigned int ret;
	once_instruction_exec(0x03,1,0,0);
	ret= jtag_data_read16();
	return ret; 
}

/* reads EOnCE control reg */
unsigned int once_ctrl_read()
{
	once_instruction_exec(0x01,1,0,0);
	return jtag_data_read8();
}

/* writes EOnCE control reg */
void once_ctrl_write(unsigned int data)
{
	once_instruction_exec(0x01,1,0,0);
	jtag_data_write8(data);
}

/* writes data into upper ORx register of the target */
#define once_tx_upper_data(data) once_instruction_exec(0x0d,0,0,0);jtag_data_write16(data)

/* reads data from upper OTx register of the target */
#define once_rx_upper_data() (once_instruction_exec(0x09,1,0,0),jtag_data_read16())

/* reads data from lower OTx register of the target */
#define once_rx_lower_data() (once_instruction_exec(0x07,1,0,0),jtag_data_read16())

/* reads status of OTx & ORx registers */
#define once_data_status() (once_instruction_exec(0x06,1,0,0),jtag_data_read8())

/* reads Eonce base address - XXX this returns only the top 8-bits */
unsigned int once_read_base_addr() 
{
	unsigned int ret;
	once_instruction_exec(0x05,1,0,0);
	ret = jtag_data_read8();
	return ret;
}

//--------------------------------------------------------------------------------------------------------

unsigned long readMemory(unsigned long addr,int is_p)
{
	unsigned long ret;
	once_move_long_to_r0(addr);
	once_move_long_to_r2((unsigned long)once_reply_addr);
	if(is_p)
	{
		once_move_at_pr0_inc_to_y0();
	}
	else
	{
		once_move_at_r0_inc_to_y0();
	}
	once_nop();
	once_move_y0_at_r2();
	ret = once_rx_upper_data();
	return ret;
}

void writeMemory(unsigned long addr,int is_p, unsigned int val)
{
/* move.l #addr,r0 */
	once_move_long_to_r0(addr);
/* move #val,y0 */
	once_move_value_to_y0(val);
	if(is_p)
	{
		once_move_y0_at_pr0_inc();
	}
	else
	{
		once_move_y0_at_r0_inc();
	}
	once_nop();
}

//--------------------------------------------------------------------------------------------------------

void gotoSelectDR()
{
	int i;
	

	JTAG_TMS_SET;
	// This will put it into debug mode, i guess.
	for (i=0;i<10;i++) 
	{
		JTAG_TCK_RESET;	     
		JTAG_TCK_SET;
	}
	JTAG_TMS_RESET;
	JTAG_TCK_RESET;      /* Go to Run-Test-Idle */
	JTAG_TCK_SET;
	JTAG_TMS_SET;        /* Go to Select-DR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
}


//--------------------------------------------------------------------------------------------------------


/* measures data and instruction JTAG path lengths */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
/* returns 0 in case measurement has overflown, 1 in case measurement is OK */
int jtag_measure_paths(void) 
{
	unsigned int i;

	SetLEDs(0x01,0);

	if(curState != 0x07)
	{
		
		getPush(0x3f,0x1);
		showData(curState,4);	
	}

	JTAG_TMS_SET;								/* Go to Select-IR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TMS_RESET;								/* Go to Capture-IR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-IR */ /* Now the Jtag is in the Shift-IR state */
	JTAG_TDI_ASSIGN(1);
	for (i=0;i<JTAG_PATH_LEN_MAX;i++) 
	{
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
	}
	JTAG_TDI_ASSIGN(0);								/* shift 0 into beginning of the IR path */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TDI_ASSIGN(1);
	i=0;
	isReading=1;
	SetLEDs(0x03,0);
	while ((JTAG_TDO_VALUE)&&(i<JTAG_PATH_LEN_MAX)) /* wait for the 0 to appear at the path end */

	{ 
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		i++;
	}
	isReading=0;	
	JTAG_TCK_RESET;
	JTAG_TCK_SET;									/* the whole IR path now contains 1 = BYPASS command */
	JTAG_TMS_SET;
	JTAG_TCK_RESET;									/* Go to Exit1-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;									/* Go to Update-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;									/* Go to Select-DR-Scan */
	JTAG_TCK_SET;
	instr_pl=i;
	SetLEDs(0x07,0);


	JTAG_TMS_RESET;								/* Go to Capture-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Now the Jtag is in the Shift-DR state */
	JTAG_TDI_ASSIGN(1);
	for (i=0;i<JTAG_PATH_LEN_MAX;i++) 
	{
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
	}
	JTAG_TDI_ASSIGN(0);								/* shift 0 into beginning of the DR path */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TDI_ASSIGN(1);
	i=0;
	isReading=1;
	SetLEDs(0x0f,0);
	while ((JTAG_TDO_VALUE)&&(i<JTAG_PATH_LEN_MAX)) /* wait for the 0 to appear at the path end */

	{ 
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		i++;
	}
	isReading=0;
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* the whole DR path now contains 1 */
	JTAG_TMS_SET;
	JTAG_TCK_RESET;								/* Go to Exit1-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Update-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Select-DR-Scan */
	JTAG_TCK_SET;
	data_pl=i;

	SetLEDs(0x1f,0);
	if ((data_pl<JTAG_PATH_LEN_MAX)&&(instr_pl<JTAG_PATH_LEN_MAX))
		return(1); 
	else
		return(0);
}

/* shifts up to 32 bits in and out of the jtag DR path */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
unsigned long jtag_data_shift(unsigned long data, unsigned int bit_count) 
{
	int i;
	unsigned long result=0;
	unsigned long curBit=1;

	if(curState != 0x07)
	{
		
		getPush(0x3f,0x2);
		showData(curState,4);	
	}


	JTAG_TMS_RESET;								/* Go to Capture-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Go to Shift-DR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;								/* Now the Jtag is in the Shift-DR state */
	isReading=1;
	for (i=0;i<bit_count;i++) 
	{
		JTAG_TDI_ASSIGN(data);
		data>>=1;
		if ((data_pp==0)&&(i==(bit_count-1))) JTAG_TMS_SET;	/* Go to Exit1-DR */
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		if(JTAG_TDO_VALUE)
		{
			result|=curBit;
		}
		curBit=curBit + curBit;
	}
	isReading=0;
	JTAG_TCK_RESET;								/* Go to Update-DR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET;								/* Go to Select-DR-Scan */
	WAIT_100_NS;
	WAIT_100_NS;
	JTAG_TCK_SET;
	JTAG_TDI_ASSIGN(0); // no longer important
	return(result);
}

/* Executes Jtag command */
/* expects Select-DR-Scan state of the Jtag state machine state upon entry */
/* and leaves the Jtag in Select-DR-Scan on exit */
/* the second parameter specifies instruction length in bits */
unsigned int jtag_instruction_exec(unsigned int instruction, unsigned int instr_len) 
{
	unsigned int i,status=0;

	if(curState != 0x07)
	{
		
		getPush(0x3f,0x4);
		showData(curState,4);	
	}

	JTAG_TMS_SET; 								/* Go to Select-IR-Scan */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TMS_RESET; 								/* Go to Capture-IR */
	JTAG_TCK_RESET;
	JTAG_TCK_SET;
	JTAG_TCK_RESET;
	JTAG_TCK_SET; 								/* Go to Shift-IR */
	/* Now the Jtag is in the Shift-IR state */
	isReading=1;
	for (i=0;i<instr_len;i++) 
	{
		JTAG_TDI_ASSIGN(instruction);
		instruction>>=1;
		if ((instr_pp==0)&&(i==(instr_len-1))) JTAG_TMS_SET; 	/* Go to Exit1-IR */
		JTAG_TCK_RESET;
		JTAG_TCK_SET;
		status>>=1;
		status|=(JTAG_TDO_VALUE<<3);
	}
	isReading=0;
	JTAG_TCK_RESET; 								/* Go to Update-IR */
	JTAG_TCK_SET;
	JTAG_TCK_RESET; 								/* Go to Select-DR-Scan */
	JTAG_TCK_SET;
	return(status);
}


/* selects the CORE TAP Controller by modifying the TLM register */
/* to be used when Master TAP is selected */
unsigned long select_core_tap(void) 
{
	unsigned int cmdRes;
	unsigned long dataRes;

	if(instr_pl != 8)
	{
		showData(instr_pl | 0x20,8);
		return 0;
	}

	cmdRes  = doExecute("X`: SELECT_TLM (from master)",0x05,8);	/* select TLM */
	dataRes = doData(   "D: choose TAP 2 - core",0x2,4);		/* select core TAP */

	jtag_measure_paths();					/* rescan jtag chain (Core TAP and Master TAP have different intruction lengths) */

	if(instr_pl != 4 ) // || dataRes != 2)
	{
		showData((instr_pl << 8) | dataRes,8);
		return 0;			
	}
	return 2;
}

/* selects the Master TAP Controller by modifying the TLM register */
/* to be used when Core TAP is selected */
unsigned long select_master_tap(void) 
{
	unsigned int cmdRes;
	unsigned long dataRes;

	SetLEDs(0x11,5000);
	SetLEDs(0x22,5000);
	SetLEDs(0x04,5000);
	SetLEDs(0x28,5000);
	SetLEDs(0x18,5000);

	if(instr_pl != 4)
	{
		showData(instr_pl | 0x80,8);
		return 0;
	}
	cmdRes  = doExecute("X: SELECT_TLM (from core)",0x08,4);	/* select TLM */
	dataRes = doData("D: choose TAP 1 - master",0x1,4);			/* select master TAP */
	jtag_measure_paths();										/* rescan jtag chain (Core TAP and Master TAP have different intruction lengths) */
	if(instr_pl != 8 ) // || dataRes != 1)
	{
		showData((instr_pl <<8)| dataRes,8);
		return 0;			
	}
	return 1;
}

unsigned int doExecute(char *n,unsigned int  cmd,unsigned int bit_length)
{
	unsigned int code=0;
	if(n)n=0;
	SetLEDs(0x0a,0);
	code = jtag_instruction_exec(cmd,bit_length);
	SetLEDs(0x0b,0);
	return code;
}


unsigned long doData(char *n,unsigned long data,unsigned int bit_length )
{
	unsigned long result=0;
	if(n)n=n;
	SetLEDs(0x08,1000);
	result = jtag_data_shift(data,bit_length);
	SetLEDs(0x09,1000);
	return result;
}


//======================--------------------------------------------------------

void clearBits();
void clearBits()
{
	Bit_TCK_SetVal();
	Bit_TMS_SetVal();
	Bit_TDI_ClrVal();
	clearedBits=1;
}

int gotInterrupted()
{
	int ret=0;
	if(intFlag != 0)
	{
		ret=1;	
		intFlag=0;
	}
	return ret;
}




int doIDCODE(unsigned long id,unsigned long mask,int iLen)
{	
	unsigned int cmdRes;
	if(instr_pl != iLen)
	{
		sysRunState= 2;
		return 0;	
	}
	// 0x02211004
	// 0x01f2601D (seems to be 0x01c0601D ?)
	cmdRes=doExecute("X: IDCODE",IDCODE,instr_pl);

	tapID = doData("D: IDCODE",0x450A0804,32);

	showData(tapID,32);

	if((tapID & mask) != id)
	{
		if(tapID == 0) 
		{
			sysRunState=4;
		}
		else
		{
			sysRunState=5;
		}
		return 0;
	}
	return 1;
}

int doInitTap()
{
	unsigned int cmdRes;
	unsigned int i;
	
	gotoSelectDR();

	if(!jtag_measure_paths())
	{
		sysRunState=4;
		return -1;
	}
	


	if(instr_pl != 4)
	{
		if(instr_pl != 8)
		{
			sysRunState=5;	
			return -1;
		}

		if(select_core_tap() == 0)
		{
			sysRunState=6;
			if(instr_pl != 4)
			{
				sysRunState|=0x10;
			}
			
			return -1;
		}


		for(i = 0; i<10; i++)
		{

			cmdRes = doExecute("doExecute DBG_REQ",DEBUG_REQUEST,4);
			if(cmdRes == 0x0d)break;
		}
		

		if(cmdRes != 0x0d)
		{
			showData(cmdRes,8);
			sysRunState=7;
			return -1;
		}

		if(select_master_tap() == 0)
		{
			sysRunState=8;
			return -1;
		}
		
		if(!doIDCODE(0x601D,0xffff,8))
		{
			// correct sysRunState set already; no need add bit for 'in core'
			return -1;	
		}
	}

	if(select_core_tap() == 0)
	{
		sysRunState=9;
		return -1;
	}

	if( instr_pl != 4 )
	{
		sysRunState=10;
		return -1;
	}

	if(!doIDCODE(0x02211004,0xffffffff,4))
	{
		sysRunState|=0x10;
		// sysRunState is already set 
		return -1;	
	}

	
	Cpu_Delay100US(10);

	
	for(i = 0; i<5; i++)
	{
		cmdRes = doExecute("X: Enable OnCE",ENABLE_ONCE,4);            
		if(cmdRes == 0x0d)break;
	}

	if(cmdRes != 0x0d)
	{
		sysRunState=11;
		return -1;
	}

	
	once_base = once_read_base_addr();
	once_reply_addr = ((unsigned long)(once_base)<<16)+0xffff;
	once_move_long_to_r0(once_reply_addr);
	once_move_omr_to_y0();			/* read OMR */
	once_move_y0_at_r0();    
	i = once_rx_upper_data();

	if ((i&0x01) == 1) 
	{
		sysRunState=12; // XXX is this a problem?
		return -1;
	}
	return 0;
}


void loop() 
{
	if (intFlag) 
	{
		if (sysRunState == 0) 
		{
			gotInterrupted();
			tapID=0;
			clearedBits=0;
			sysRunState=1;
			Cpu_Delay100US(100);

			if(doInitTap() != 0)return;
			//once_ctrl_write(0x20); // enable DEBUGHLT - needed later
			{
				unsigned char z=0;
				unsigned int mem[0x11];
				unsigned long val;
				unsigned int addr;

				for(addr=0;addr<0x10;addr++)mem[addr]=0;
				for(addr=0x0000;addr<0x0100;addr+=0x1)
				{
					writeMemory(addr,0,addr);
				}
				for(addr=0x0000;addr<0x0400;addr+=0x10) // all of RAM
				{
					int j;
					mem[0x10]=0xffff;
					for(j = 0;j<0x10;j++)
					{					
						if(intFlag)
						{
							sysRunState=1;

							return;
						}
						val=readMemory(addr + j,0);
						mem[j]=(unsigned int)val;
						
					}
					z=!z;
				}                
			}
			//once_ctrl_write(0x0);
			once_stop_debugging_and_reset();
			doExecute("X: IDCODE",IDCODE,4);
			tapID = doData("D: IDCODE",0,32);
		}
		return;
	}
	if(sysRunState==0)
	{
		static int pattern=1;
		static int patternDir=1;
		if(patternDir==1)
		{
			if(pattern<0x20)
			{
				pattern<<=1;	
			}
			else
			{
				patternDir=-1;
			}
		}
		else
		{
			if(pattern > 1)
			{
				pattern>>=1;
			}
			else
			{
				patternDir=1;
			}
		}
		SetLEDs(pattern,1000);
	}
	else
	{
		SetLEDs(0x20,1000);
		SetLEDs(0x3f,1000);
		SetLEDs(sysRunState | 0x20,7000);
		SetLEDs(0x3f,1000);
		SetLEDs(0x20,1000);
		if(intFlag)
		{
			
			sysRunState=0;
			intFlag=0;
			return;	
		}
	}
}



/* END jtag_stuff */



void main(void)
{
	/* Write your local variable definition here */
	int i;

	/*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  
	PE_low_level_init();
	/*** End of Processor Expert internal initialization.                    ***/
  
  
  
	/* Write your code here */

	for(i=0;i<0x3f;i++)
	{
		SetLEDs(i,500);	
	}
	for(;;) 
	{
		if((!clearedBits)&&(sysRunState != 0))
		{
			clearBits();
			clearedBits=1;
		}
		loop();
	}
}

/* END self_JTAG */
/*
** ###################################################################
**
**     This file was created by Processor Expert 3.00 [04.35]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
