/** ###################################################################
**     Filename  : self_JTAG.C
**     Project   : self_JTAG
**     Processor : MC56F8006_32_LQFP
**     Version   : Driver 01.14
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/5/2010, 1:49 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
** ###################################################################*/
/* MODULE self_JTAG */


#define __VERSION_NUMBER__ 0x3


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "Bit_TDI.h"
#include "Bit_TDO.h"
#include "Bit_TCK.h"
#include "Bit_TMS.h"
#include "DC18_LCD.h"  
#include "Inhr1.h"
#include "Mem.h"
#include "LED0.h"
#include "LED1.h"
#include "SOUT1.h"
#include "PWREN.h"
#include "LCD_CS.h"
#include "LCD_SIMO.h"
#include "LCD_SCLK.h"
#include "LCD_DC.h"
#include "LCD_RST.h"
#include "LCD_BUSY.h"
#include "SW0.h"
#include "SW1.h"
#include "Term.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "self_JTAG.h"


#define CUR_MAGIC 0x5a50
#define GEN_SIZE 0x80

#define genFlash   (0x1e00)
#define curFlashed (0x1d00)

int intFlag;

int systemRunState= 0;

word lfsr = 0xACE1u;
word lfsr2 = 0x0123u;

word curGenIdx;
static void initCurGenIdx()
{	
	asm(move.w #(GEN_SIZE-2),curGenIdx);

//	curGenIdx=(GEN_SIZE-2); // the first word is the version.
	return;
}

word check_for_patch(dword data_offset,word curWord)
{

	if(data_offset == (genFlash + curGenIdx -1))
	{
		curWord=0; // 	new generation,
	}

	if(data_offset == (genFlash + curGenIdx ))
	{
		curWord=1 + pmem_read((word*)curFlashed);
	}

	if(data_offset == curFlashed)
	{
		curWord=0; // 	reset the total number,
	}
	

	if(data_offset == (dword) (&initCurGenIdx) + 2)
	{
		if(curGenIdx>1)
		{
			curWord=(curGenIdx-1);
		}
	}

	if(data_offset == (dword) (&lfsr))
	{
		if(curGenIdx>1)
		{
			curWord=lfsr;
		}
	}
	
	if(data_offset == (dword) (&lfsr2))
	{
		if(curGenIdx>1)
		{
			curWord=lfsr2;
		}
	}
	
	return curWord;
}

static void CountBitLED(int n,word l)
{
	int i,v=0;
	int p=3;
	static int old;
	
	for(i=0;i<6;i++)
	{
		if(n&1)v++;
		n>>=1;
	}

	SetLEDs((1<<v)-1,l);
	old = v;
}

static word do_lfsr(word tapMask,word cur_reg)
{
// based on http://en.wikipedia.org/wiki/Linear_feedback_shift_register
/* tapMask = 1 at bits corresponding to taps, 0 elsewhere. */
		
		word lsb = cur_reg & 1;			/* Get lsb (i.e., the output bit). */
		cur_reg >>= 1;					/* Shift register */

		/* Only apply toggle mask if output bit is 1. */
		if (lsb == 1) cur_reg ^= tapMask;	

		return cur_reg;
}

void sparkleTime()
{
	/* taps: 16 14 13 11; characteristic polynomial: x^16 + x^14 + x^13 + x^11 + 1 */
	byte a;
	for(a=0;a<6;a++)
	{
		if((lfsr&0x1ff) == 0x001u)
		{
			Term_SendStr("say my name!");
			say_my_name();
		}
		lfsr = do_lfsr(0xB400u,lfsr );
		lfsr2= do_lfsr(0x6000u,lfsr2);
	}
	SetLEDs((int)(lfsr ^ lfsr2),200+ (lfsr&0x1ff));	
	CountBitLED((int)(lfsr ^ lfsr2),200 + (lfsr2&0x1ff));
}


asm void Cpu_DelayQuick(word us1)
{
  /* Total irremovable overhead: about 16 cycles */
  /* move.w: 2 cycles overhead (load parameter into register) */
  /* jsr:    5 cycles overhead (jump to subroutine) */
  /* rts:    8 cycles overhead (return from subroutine) */
  /* nop:    1 cycles overhead (aditional nops) */

  loop:
  /* 1 us delay block begin */
  adda #2, SP                          /* (1 c: 31.25 ns) move SP forward */
  move.l A10, X:(SP)                   /* (2 c: 62.5 ns) push A */
  move.w #32, A                       /* (2 c: 62.5 ns) number of iterations */
  do A, label0                         /* (8 c: 250 ns) repeat 3178x nop */
	nop                                /* (1 c: 31.25 ns) wait for 1 c */
  label0:
  move.l X:(SP), A                     /* (2 c: 62.5 ns) pop A */
  suba #2, SP                          /* (1 c: 31.25 ns) move SP back */
  /* 10 us delay block end */
  dec.w Y0                             /* us10 parameter is passed via Y0 register */
  jne loop                             /* next loop */
  nop                                  /* avoid pipeline conflicts */
  rts                                  /* return from subroutine */
}

static void doSetLED(int val,word t)
{

	word offval;
	word onval;
	offval=((word)~val)&0x3;
	onval =((word	) val)&0x3;


	clrSetReg16Bits(GPIO_A_DR,offval,onval);
	if(intFlag)return;
	if(t==0) return;
	Cpu_Delay100US(t);
	
} 	

word ledState[8];
word ledIdx;

void SetLEDs(int val,word t)
{
	word offval;
	word onval;

	ledState[ledIdx++]=(word)val;
	ledIdx&=0x7;
	
#if 0
	offval=((word)~val)&0x3;
	onval =((word	) val)&0x3;

#else	

	offval=((word)~val)&0x3f;
	onval =((word	) val)&0x3f;
#endif

	clrSetReg16Bits(GPIO_A_DR,offval,onval);
	if(intFlag)return;
	if(t==0) return;
	Cpu_Delay100US(t);
}

static void BlinkLEDs(int v1,word t1,int v2,word t2)
{
	SetLEDs(v1,t1);
	SetLEDs(v2,t2);
}

static void multiBlinkLEDs(int v1,word t1,int v2,word t2,word count)
{
	word j;
	for(j=0;j<count;j++)
	{
		BlinkLEDs(v1,t1,v2,t2);
	}
}

void showBadness(int p,int q)
{
	int i;
	Term_SendStr("Bad: ");
	showData((dword)p,8);
	showData((dword)q,8);
	return;
	for(i=2;i<11;i++)
	{
//		BlinkLEDs(p,(word)(20000/i),q,(word)(20000/i));
	}
	SetLEDs(0,5000);
}

void showData(dword data,word bit_length)
{
	int i;
	int bl;

	bl=(int)bit_length;
	Term_SendStr(" 0x");
//	SetLEDs(0x30,8000);
	for(i=bl-4; i>=0;i-=4)
	{
		word p;
		p=(word)( (data>>i) & 0xf );
		Term_SendChar((unsigned char)("0123456789abcdef"[p]));

//		multiBlinkLEDs(0x36,6000,0x30,2000,p);
//		SetLEDs(0x30,8000);				
	}
//	SetLEDs(0x10,2000);
	for(i=0;i<6;i++)
	{
//		BlinkLEDs((1<<i),500,(0x20>>i),200);
	}
}

//------------------------------------------------------------------------------------
//======================--------------------------------------------------------------
//------------------------------------------------------------------------------------
//====================================================================================


static void incrementCurFlashedCount()
{
	word i;
	int res;
	
	i=pmem_read((word*)(curFlashed));
	setRegBits(FM_USTAT,FM_USTAT_BLANK_MASK);
	setRegBits(FM_USTAT,FM_USTAT_PVIOL_MASK);
	setRegBits(FM_USTAT,FM_USTAT_ACCERR_MASK);
	
	res=Mem_SetWordFlash(Mem_DataPtr2Addr(curFlashed),i+1);
}

static void dashLED()
{
	word i;
	for(i=0;i<128;i+=2)
	{
		BlinkLEDs(0xc,i,0,128-i);
	}
}

static void say_my_name()
{
	word m;
	word d;
	dashLED();
	for(m=GEN_SIZE -1; m>curGenIdx;m--)
	{
//		word i;
		d=pmem_read((word*)genFlash+m);
		if(d!=0xffff)
		{
			Term_SendNum(d);
//			SetLEDs(0x00,8000);
//			for(i=10;i<d;i+=10)
//			{
//				multiBlinkLEDs(0x0c,60,0x00,10,50);
//				SetLEDs(0x00,3000);
//			}
//			d=d%10;
//			SetLEDs(0x00,3000);
			
//			for(i=0;i<d;i++)
//			{
//				multiBlinkLEDs(0x0c,20,0x00,50,50);				
//				SetLEDs(0x00,3000);
//			}
//			SetLEDs(0x00,4000);	
//			dashLED();
			Term_SendChar('.');
		}
	}
	d=pmem_read((word*)genFlash+curGenIdx);
	if(d!=0xffff)
	{
		Term_SendNum(d);
	}
	Term_SendStr("\r\n");
//	multiBlinkLEDs(0x0c,1000,0x00,3000,d);
//	SetLEDs(0x00,4000);	
//	dashLED();
}

void loop() 
{
	if (intFlag) 
	{
		if (systemRunState == 0) 
		{
			int res;
			intFlag = 0;
			clearedBits=0;
			systemRunState = 1;

			res=doInitTap();

			if(res != 0)
			{
				Term_SendStr("init fail\r\n");
				return;
			}

			if(once_set_hfmdiv())
			{
				Term_SendStr("hmfdiv failed\r\n");
				systemRunState=13;
				return;
			}
			
			if(once_flash_erase())
			{
				Term_SendStr("erase failed\r\n");
				systemRunState=14;
				return;
			}

			incrementCurFlashedCount(); // count here, incase it crashes to prevent dups if it crashes
			if(once_infect_other_badge())
			{
				Term_SendStr("inf\r\n");
				systemRunState=15;
				return;
			}
			
			jtag_disconnect();
			Term_SendStr("done\r\n");
			systemRunState=0;
		}
		else
		{
			
		}
		return;
	}
	if(systemRunState==0)
	{
		sparkleTime();
	}
	else
	{
		Term_SendStr("ret = ");
		Term_SendNum(systemRunState);
		Term_SendStr("\r\n");
		SetLEDs(0x20,1000);
		SetLEDs(0x3f,1000);
		SetLEDs(systemRunState | 0x20,7000);
		SetLEDs(0x3f,1000);
		SetLEDs(0x20,1000);
		if(intFlag)
		{
			Term_SendStr("RST\r\n");
			systemRunState=0;
			intFlag=0;
			return;	
		}
	}
}

/* END jtag_stuff */


#define	FLASH_PROT_VALUE_ADDR	0x1FFA
#define	FLASH_SECHI_VALUE_ADDR	0x1FF8
#define	FLASH_SECLO_VALUE_ADDR	0x1FF7

// apparently FM_PROTB exists on some DSP800E based chips,
// but not on 56F8006. Zero to be consistent.
#define FM_PROTB				(*((volatile word *)0x0000F411))	

static void make_flash_writable()
{
	doSetLED(1,1000);
	clrRegBits(FM_CNFG,FM_CNFG_LOCK_MASK);
	setReg16(FM_PROT,0);
	setReg16(FM_PROTB,0); 	 
	setRegBits(FM_USTAT,FM_USTAT_PVIOL_MASK);
	setRegBits(FM_USTAT,FM_USTAT_ACCERR_MASK);
	setRegBits(FM_USTAT,FM_USTAT_BLANK_MASK);

	Mem_SetWordFlash(FLASH_PROT_VALUE_ADDR ,0); 
	Mem_SetWordFlash(FLASH_SECHI_VALUE_ADDR,0);
	Mem_SetWordFlash(FLASH_SECLO_VALUE_ADDR,0);
	
	if(pmem_read((word*)genFlash + GEN_SIZE-1) != __VERSION_NUMBER__)
	{	
		Mem_SetWordFlash(genFlash + GEN_SIZE-1, __VERSION_NUMBER__);
		Mem_SetWordFlash(genFlash + GEN_SIZE-2, 1);
		Mem_SetWordFlash(genFlash + GEN_SIZE-3, 2);
	}
}

static void GenData(byte bg)
{
  uint8_t colAddr, page;  
  uint16_t i;  
  uint8_t pData;
  int ii,idx;
  word v;

  // Segment remapping must be done when loading data.  
  #ifdef REMAP_IMAGE  
    PutCharSPI( 0xA1, CMD_MASK );       // Remap segs.  
  #else  
    PutCharSPI( 0xA0, CMD_MASK );       // Don't remap segs.  
  #endif  

  idx=0;
  ii=0;      
  // Loop through the data to write.  
  for (i = 0; i < BYTES_PER_IMAGE; i++)  
  {    
 	ii++;
 	if(ii==N_ROWS/8)
 	{
 		idx++;
 		ii=0;
 	}
    colAddr = (uint8_t)((i % N_COLS) + 2); // Add two to skip unused & frame.  
  
    // Set page and column address as needed.  
    if ((colAddr == 2) || (i == 0) )  
    {  
      page = (uint8_t)((i/N_COLS) + 2); // Add two to skip unused & frame.  

      PutCharSPI( (uint8_t)(COL_MSN | (colAddr / 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(COL_LSN | (colAddr % 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(PAGE_ADD | page) , CMD_MASK );  
    }  
  
  	// 
 	v=pmem_read((unsigned int*)genFlash + ii);

  	pData=bg;
  	if(idx < v)
  	{  		
	  	if(v==0xffff) pData=0xfe;
	  	else
	  	pData=(byte)"\x00\x01\x03\x07\x0f\x1f\x3f\x7f"[v%8];
  	}
    // Write a data byte.  
    PutCharSPI( pData, DATA_MASK );  
  }  
  
  // NOP command is required after writing data before chip select goes high.  
  PutCharSPI( 0xE3, CMD_MASK );  
}  

void main(void)
{
	/* Write your local variable definition here */
	/*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
	PE_low_level_init();
	/*** End of Processor Expert internal initialization.                    ***/

	/* Write your code here */
	
	Cpu_DisableInt();
	HardResetDisplay();
	Cpu_EnableInt();
	
	Cpu_Delay100US(10000);
	GenData(0xff);
	Display(0,N_ROWS);
	Cpu_Delay100US(10000);
	LoadBorder(1);

	Term_SendStr("boot\r\n");
#if 1
	clrReg16Bits(GPIO_A_DR,  0x3fU);
	setReg16Bits(GPIO_A_DDR, 0x3fU); // GPIO_A[0-5].direction = output;
	clrReg16Bits(GPIO_A_PER, 0x3fU); // GPIO_A[0-5].enable_peripheral = no... so GPIO	
#else
	clrReg16Bits(GPIO_A_DR,  0x03U);
	setReg16Bits(GPIO_A_DDR, 0x03U); // GPIO_A[0-5].direction = output;
	clrReg16Bits(GPIO_A_PER, 0x03U); // GPIO_A[0-5].enable_peripheral = no... so GPIO	
#endif
	
	initCurGenIdx();
	make_flash_writable();
	
	BlinkLEDs((int)curGenIdx,4000,0,1000);

	Term_SendStr("fls\r\n");
	
	if(pmem_read((word*)genFlash + curGenIdx) == 0xffff)
	{
		incrementCurFlashedCount();
	}
	
	intFlag=0;
		
	for(;;) 
	{
		if((!clearedBits)&&(systemRunState != 0))
		{
			Term_SendStr("clrBts\r\n");
			clearBits();
		}
		loop();
	}
}

/* END self_JTAG */
/*
** ###################################################################
**
**     This file was created by Processor Expert 3.00 [04.35]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
