/** ###################################################################
**     Filename  : self_JTAG.C
**     Project   : self_JTAG
**     Processor : MC56F8006_32_LQFP
**     Version   : Driver 01.14
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/5/2010, 1:49 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
** ###################################################################*/
// 
// What do you mean "No public methods"? Releasing this source makes
// all our previously secret methods public...
// 
// What? It's to do with what now?
// Oh... So something to do with like C++ or something?
// Ah... Ok, sure. 
// 
// Computers are hard.
// 
// -- Team Redacted

/* MODULE self_JTAG */

// Final 1.15
// -----------
// - Added random inversion of output.
// - Fixed LFSR random bit usage in most places: should never reuse bits.
// - Turned off LEDs when entering sleep.
// - Display unique pattern and pause a little, after pushing the button so you can 
//   see it did something,even if it fails immediately after. 
// - Fixed check if 'infection (attempt) count' was initialized (was checking wrong place)
//   - Actually display it even if it is 0
// - Made both buttons behave consistently 
// 
// Final 1.12
// -----------
// - Fixed reboot for real by adding pullup to GPIO_A7 before making it a peripheral again 
// - Modified 'redaction effect'.
// 
// Final 1.11
// -----------
// - Added extra graphics, sped up some display code.
// - Added 'redaction effect' to title text.
// - Fixed reboot when button is pushed, (attempted - failed)
// 
// Final 1.1
// -----------
// Original 'final' version. LCD output
// 

// Sorry about the variable/function naming convention inconsistency.
// this is what happens when you copy and paste, then search and replace, and then give up.
//
// Any other things that you don't like are intentional.
// 
// Maybe.
// 
// -- Team Redacted


// !!!! ENDUSER !!!!
// change this to a random number... in the range 0x0001 ... 0xfffe 
#define __VERSION_NUMBER__ 4

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "Bit_TDI.h"
#include "Bit_TDO.h"
#include "Bit_TCK.h"
#include "Bit_TMS.h"
#include "DC18_LCD.h"  
#include "Inhr1.h"
#include "TI1.h"
#include "AD1.h"
#include "Mem.h"
#include "LED0.h"
#include "SOUT1.h"
#include "PWREN.h"
#include "LCD_CS.h"
#include "LCD_SIMO.h"
#include "LCD_SCLK.h"
#include "LCD_DC.h"
#include "LCD_RST.h"
#include "LCD_BUSY.h"
#include "SW0.h"
#include "SW1.h"
#include "Term.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "self_JTAG.h"


#include "vir.h"

// Random foward definitions are encouraged by the C standard.
// If you don't believe me, have a look. It's in there somewhere
// in a section which doesn't seem to have anything to do with
// forward declarations.
void FillDisplay(byte z,byte z2);

#define GEN_SIZE 0x80
// arbitrary limit of 1/2 of a page of flash. (0x80 *2 == 0x100 == flash page size in words)
// if the datatype of generation changed from word to dword would,
// then 0x80 would be an entire page.
//
// The only reason to care about pages is... um... give me a moment...
// I'll come up with one eventually. 
//
// Oh yeah.. historic reasons. Sorry.


// make sure the time is good so we don't get instant returns
static void wakeupTime(word timeout)
{
	if(timeout < 260)  timeout = 260;
	if(timeout > 16320) timeout = 16320;
  	TI1_SetPeriodMS(timeout);
}

// wrapper.. #define ?
// This starts the Timer Interrupt with the time set in wakeupTime() 
static void setWakeup()
{
	TI1_Enable();
}

// wrapper.. #define  ?
// This makes sure the timer stops when we don't need it.
static void clrWakeup()
{
	TI1_Disable();
}
 
// turns off all the bits, makes them all inputs, and disables the pullup resistors... 
// one would think this should reduce the power consumption somewhat. or at least
// not make it any worse.
static void JTAG_down()
{
  clrReg16Bits(GPIO_D_DR,     0x0f);         
  clrReg16Bits(GPIO_D_DDR,    0x0f); 
  clrReg16Bits(GPIO_D_PUR,    0x0f);
}

// wakes up the JTAG connector and sets the stuff to what PE_low_level_init does
static void JTAG_up()
{
  setReg16Bits(GPIO_D_IESR,   0x0f);       
  clrReg16Bits(GPIO_D_DR,     0x0d);         
  clrSetReg16Bits(GPIO_D_DDR, 0x02, 0x0d); 
  clrReg16Bits(GPIO_D_PER,    0x0f);        
  clrReg16Bits(GPIO_D_PUR,    0x02);
}

// from badge code
// this does the go to low-power mode part.
// I may have other things I need to do here due to JTAG...
// but I think I caught them all.

static void dc18_sleep(void)
{
	// set GPIOA6/TXD to input before going to sleep to prevent back-powering the FT232RL

	GPIO_A_PUR &= ~GPIO_A_PUR_PU6_MASK;  // disable A6 pull-up
	GPIO_A_DDR &= ~GPIO_A_DDR_DD6_MASK;  // set as input
	GPIO_A_PER &= ~GPIO_A_PER_PE6_MASK;  // set to GPIO mode


	OCCS_DIVBY |= OCCS_DIVBY_COD3_MASK;  // Divide peripheral clock by 256 (1xxx) (15.625kHz @ 8MHz ROSC, ~781Hz @ 400kHz ROSC)

	PMC_SCR |= PMC_SCR_LPR_MASK;         // Request low-power run/wait mode
	ZeroSPILines();
	clrReg16Bits(GPIO_A_DR,3);
	JTAG_down();
	setWakeup(); // set the timer interrupt
	
    //    Before WaitMode
	////////////////////////////////////////////////////////////////////////////////
	Cpu_SetWaitMode();					// Awake on the next external interrupt from SW0 or SW1
										// or PIT1. Don't forget that... please... at least. 
	////////////////////////////////////////////////////////////////////////////////
    //    After WaitMode
	
	clrWakeup(); // kill the timer interrupt
	JTAG_up();

	PMC_SCR &= ~PMC_SCR_LPR_MASK;                                 // Disable low-power run/wait mode

	OCCS_DIVBY &= ~OCCS_DIVBY_COD3_MASK; // Restore original peripheral clock divider

	// re-enable GPIOA6/TXD to back-power the FT232RL to allow us to read the correct #PWREN value
	// (to determine if USB is connected or not)


	GPIO_A_PER |= GPIO_A_PER_PE6_MASK;   // set to peripheral mode


}

byte invertIt=1;

// "Team Redacted"
byte __pmem teamredacted[64] = {
  0x02, 0x02, 0xfe, 0x02, 0x02, 0x70, 0xa8, 0xa8, 
  0xb0, 0x00, 0xc8, 0xa8, 0xa8, 0xf0, 0x00, 0xf0, 
  0x08, 0x08, 0xf0, 0x08, 0x08, 0xf0, 0x00, 0x00,
  0x00, 0x00, 0xfe, 0x12, 0x12, 0x32, 0xcc, 0x00, 
  0x70, 0xa8, 0xa8, 0xb0, 0x00, 0x70, 0x88, 0x88, 
  0xfe, 0x00, 0xc8, 0xa8, 0xa8, 0xf0, 0x00, 0x70,
  0x88, 0x88, 0x50, 0x08, 0xfc, 0x88, 0x00, 0x70, 
  0xa8, 0xa8, 0xb0, 0x00, 0x70, 0x88, 0x88, 0xfe
};


// "STDin-3tion ID"
byte __pmem Banner[91] ={
 0x00, 0x22, 0x67, 0x4d, 0x59, 0x73, 0x22, 0x00,
 0x01, 0x7f, 0x01, 0x00, 0x7f, 0x41, 0x63, 0x3e,
 0x1c, 0x00, 0x44, 0x7d, 0x40, 0x00, 0x7c, 0x04,
 0x7c, 0x78, 0x00, 0x08, 0x08, 0x08, 0x00, 0x41,
 0x7f, 0x41, 0x00, 0x7c, 0x04, 0x7c, 0x78, 0x00,
 0x04, 0x7e, 0x7f, 0x05, 0x01, 0x00, 0x38, 0x7c,
 0x54, 0x5c, 0x58, 0x00, 0x38, 0x7c, 0x44, 0x4c,
 0x48, 0x00, 0x04, 0x3f, 0x7f, 0x44, 0x00, 0x44,
 0x7d, 0x40, 0x00, 0x38, 0x7c, 0x44, 0x7c, 0x38,
 0x00, 0x7c, 0x04, 0x7c, 0x78, 0x00, 0x00, 0x00,
 0x00, 0x41, 0x7f, 0x41, 0x00, 0x7f, 0x41, 0x63,
 0x3e, 0x1c, 0x00
};

// where generation/tracking data is stored
// This is near the end of flash, and can be smaller as long as 
// (genFlash + curGenIdx) is greater than the end of code 
// or you could let that happen if you want... .
#define genFlash   (0x1e00) 

// where currently flashed count is stored. 
// whole page used so i don't have to memorize rest of page - RAM friendly vs FLASH friendly
#define curFlashed (0x1d00) 


// do we have a button push?
int intFlag;

// 0 == not infecting, 1 == infecting, >=2 == error code from infecting
// understanding the error code requires reading the code.
int systemRunState= 0;


// Linear Feedback Shift Registers - used for PRNG stuff
// seeded either in main, or from value at time of infection

/* taps: 16 14 13 11; characteristic polynomial: x^16 + x^14 + x^13 + x^11 + 1 */
// period == 65535
word lfsr=0xffff;


/* taps: 15 14; characteristic polynomial: x^15 + x^14 + 1 */
// period == 32767
word lfsr2=0xffff;

// which generation is this?
word curGenIdx;

// This function will get patched by 'check_for_patch' when the code is
// transmitted. It sets the curGenIdx value to the correct depth.

static void initCurGenIdx()
{	
	asm(move.w #(GEN_SIZE-2),curGenIdx);
//	== { curGenIdx=(GEN_SIZE-2); } // the first word is the version.
	return;
}

// does the memory address that is about to be written need patching from the 'real' data?
// return the value it should be.
word check_for_patch(dword data_offset,word curWord)
{

	if(data_offset == (genFlash + curGenIdx -1))
	{
		// initialize the new generation -- XXX needed?
		return 0; 
	}

	if(data_offset == (genFlash + curGenIdx ))
	{
		// Initial value could be made really big, as long as it doesn't run into
		// the code at the end... or you could let that happen, I guess.

		// patch the current infection count into the child's "generation data"
		return 1 + pmem_read((word*)curFlashed); 
		// if there are nearly 2^16, it'll get it wrong... unlikely to happen over
		// a few days unless someone is trying to be clever.
	}

	if(data_offset == curFlashed)
	{
		return 0; 
		// patch the total number already flashed to 0 in the child	
	}
	

	if(data_offset == (dword) (&initCurGenIdx) + 2)
	{
		if(curGenIdx>1)
		{
			curWord=(curGenIdx-1); 
			// Arbitrary limitation: once we get to a certain depth, 
			// stop counting new generatons, and give up making infections unique.
		}
	}

	if(data_offset == (dword) (&lfsr))
	{
		return lfsr;
		// keep track of current randomness state variable
	}
	
	if(data_offset == (dword) (&lfsr2))
	{
		return lfsr2;
		// keep track of other current randomness state variable
		// these will tend to diverge due to variations in processor speed
		// anyway, but it makes it harder to predict when the 'say_my_name' will
		// come up when you have some which were reset at around the same time.
		// initial values come from temperature if these variables contain 0
	}
	
	return curWord;
}

// code used for most PRNG stuff. 

static word do_lfsr(word tapMask,word cur_reg)
{
// based on http://en.wikipedia.org/wiki/Linear_feedback_shift_register
// tapMask = 1 at bits corresponding to taps, 0 elsewhere. 

		word lsb;
		lsb = cur_reg & 1;				/* Get lsb (i.e., the output bit). */
		cur_reg >>= 1;					/* Shift register */

		/* Only apply toggle mask if output bit is 1. */
		if (lsb == 1) cur_reg ^= tapMask;	

		return cur_reg;
}

#define LFSR(X,Y) ( X=do_lfsr(Y,X) )

static void refresh_LFSRs()
{
	word i;
	for(i=0;i<(0x10+(lfsr^lfsr))&0x1f;i++)
	{
		LFSR(lfsr, 0xB400u);
		LFSR(lfsr2,0x6000u);
	}
}

static void doDisplay(byte first,byte count)
{
	Display(first,count);
	refresh_LFSRs();
}



// based on code for Cpu_Delay100US(word) in Cpu.c
// Some delay, but less than Delay100US
asm void Cpu_DelayQuick(word us1)
{
  /* Total irremovable overhead: about 16 cycles */
  /* move.w: 2 cycles overhead (load parameter into register) */
  /* jsr:    5 cycles overhead (jump to subroutine) */
  /* rts:    8 cycles overhead (return from subroutine) */
  /* nop:    1 cycles overhead (aditional nops) */

  loop:
  /* 1 us delay block begin */
  adda #2, SP                          /* (1 c: 31.25 ns) move SP forward */
  move.l A10, X:(SP)                   /* (2 c: 62.5 ns) push A */
  move.w #32, A                        /* (2 c: 62.5 ns) number of iterations */
  do A, label0                         /* (8 c: 250 ns) repeat 3178x nop */
	nop                                /* (1 c: 31.25 ns) wait for 1 c */
  label0:
  move.l X:(SP), A                     /* (2 c: 62.5 ns) pop A */
  suba #2, SP                          /* (1 c: 31.25 ns) move SP back */
  /* 10 us delay block end */
  dec.w Y0                             /* us10 parameter is passed via Y0 register */
  jne loop                             /* next loop */
  nop                                  /* avoid pipeline conflicts */
  rts                                  /* return from subroutine */
}

void SetLEDs(int val,word t)
{
	word offval;
	word onval;
		
#if 1 // mask with 3 for badge, 0x3f for demoboard
	offval=((word)~val)&0x3;
	onval =((word	) val)&0x3;
#else	

	offval=((word)~val)&0x3f;
	onval =((word	) val)&0x3f;
#endif

	clrSetReg16Bits(GPIO_A_DR,offval,onval);
	if(intFlag)return;
	if(t==0) return;
	Cpu_Delay100US(t);
}


// show error codes from initTap stuff; historic reasons it is like this. 
// -- used to be all SetLEDs()
void showBadness(int p,int q)
{

	Term_SendStr("Bad: ");
	showData((dword)p,8);
	showData((dword)q,8);
	return;
}


// helper func for showBadness
void showData(dword data,word bit_length)
{
	int i;
	int bl;

	bl=(int)bit_length;
	Term_SendStr(" 0x");
	for(i=bl-4; i>=0;i-=4)
	{
		word p;
		p=(word)( (data>>i) & 0xf );
		Term_SendChar((unsigned char)("0123456789abcdef"[p]));
	}
}

//------------------------------------------------------------------------------------

// update how many we have infected. 
// it's actually an upper bound only, but
// does prevent duplicate IDs
static void incrementCurFlashedCount()
{
	word i;
	int res;

	Term_SendStr("f++=");
	
	i=pmem_read((word*)(curFlashed));

	Term_SendNum(i);
	Term_CRLF();
	
	setRegBits(FM_USTAT,FM_USTAT_BLANK_MASK);
	setRegBits(FM_USTAT,FM_USTAT_PVIOL_MASK);
	setRegBits(FM_USTAT,FM_USTAT_ACCERR_MASK);
	
	res=Mem_SetWordFlash(Mem_DataPtr2Addr(curFlashed),i+1);
}


byte __pmem nums[11][3]=
{
 { 0x3e, 0x41, 0x3e },
 { 0x42, 0x7f, 0x40 },
 { 0x62, 0x51, 0x4e },
 { 0x22, 0x49, 0x36 },
 { 0x18, 0x16, 0x7f },
 { 0x4f, 0x49, 0x31 },
 { 0x3e, 0x49, 0x32 },
 { 0x61, 0x19, 0x07 },
 { 0x36, 0x49, 0x36 },
 { 0x26, 0x49, 0x3e },
 { 0x00, 0x60, 0x00 }
};

// this displays bitmaps on the LCD. It also does some 
// Looks cool though, yeah?
static void RedactTeam(word Index, word NumBytes,  byte __pmem *pData,byte last)
{  
  byte colAddr, page;  
  word i;
  word q=0;  
  byte resByte,maskByte;
  
  // Segment remapping must be done when loading data.  
  #ifdef REMAP_IMAGE  
    PutCharSPI( 0xA1, CMD_MASK );       // Remap segs.  
  #else  
    PutCharSPI( 0xA0, CMD_MASK );       // Don't remap segs.  
  #endif  
      
  // Loop through the data to write.  
  for (i = Index; i < (Index + NumBytes); i++)  
  { 
    int j;   
    colAddr = (uint8_t)((i % N_COLS) + 2); // Add two to skip unused & frame.  
  
    // Set page and column address as needed.  
    if ((colAddr == 2) || (i == Index) )  
    {  
      page = (uint8_t)((i/N_COLS) + 2); // Add two to skip unused & frame.  
      PutCharSPI( (uint8_t)(COL_MSN | (colAddr / 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(COL_LSN | (colAddr % 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(PAGE_ADD | page) , CMD_MASK );  
    }  

	for(j=0;j<4;j++)LFSR(lfsr2,0x6000u); // need 4 random  bits...
	q=lfsr2 % 9;
	resByte=pData[i-Index];
	switch (q)
	{
		case 1:
		case 2:
		case 3:
			j=(int)(i+1);
		break;
		
		case 7:
		case 8:
		case 0:
			j=(int)(i-1);
		break;
		
		default:
			j=(int)i;
		break;
	}

	j-=Index;
	if(j>=(int)NumBytes)j=(int)NumBytes-1;
	if(j<0)j=0;
	maskByte=pData[j];
  	switch(q)
  	{
  		case 3:
		case 6:
		case 0:
			maskByte>>=1;
		break;

  		case 1:
		case 4:
		case 7:
			maskByte<<=1;
		break;
		
		default: break;
  	}
  	if(last)
  	{
    	// Write a data byte. 
		if(invertIt)
		{
		    PutCharSPI( resByte, DATA_MASK );  
	  	}
		else
		{
		    PutCharSPI( (byte)(~resByte), DATA_MASK );  
		}
  	}
  	else
  	{
  		// Write something based on a data byte
		resByte = (byte)(resByte & ~maskByte);
		// it really is inverted to start with... so we invert the check
		if(!invertIt)resByte=(byte)~resByte;
	    PutCharSPI( resByte, DATA_MASK );  
  	}
  }  
  
  // NOP command is required after writing data before chip select goes high.  
  PutCharSPI( 0xE3, CMD_MASK );  
}  



// draw numbers on the scren. 0xffff = '.'
static word writeNum(word num,word Index)
{
	byte colAddr, page;  
	word i,j;
	word numLen;
	word showIdx;
	// Segment remapping must be done when loading data.  
	#ifdef REMAP_IMAGE  
	PutCharSPI( 0xA1, CMD_MASK );       // Remap segs.  
	#else  
	PutCharSPI( 0xA0, CMD_MASK );       // Don't remap segs.  
	#endif  

	numLen=1;
	if(num>9)numLen++;
	if(num>99)numLen++;
	if(num>99)numLen++;
	if(num>999)numLen++;
	if(num == 0xffff)numLen=1;
	Term_SendChar('(');
	showData(num,16);
	showData(Index,16);
	Term_SendChar(')');
	Term_CRLF();
	Index=(Index+3)&0xfc;

	// Loop through the data to write. 
	for (i = Index+(numLen*4); i<BYTES_PER_IMAGE && i>0; i-=4)  
	{    

		colAddr = (uint8_t)((i % N_COLS) + 2); // Add two to skip unused & frame.  

		// Set page and column address as needed.  
		// -- Team Redacted Hack : Always set cursor. --
		page = (uint8_t)((i/N_COLS) + 2); // Add two to skip unused & frame.  
		PutCharSPI( (uint8_t)(COL_MSN | (colAddr / 16)) , CMD_MASK );  
		PutCharSPI( (uint8_t)(COL_LSN | (colAddr % 16)) , CMD_MASK );  
		PutCharSPI( (uint8_t)(PAGE_ADD | page) , CMD_MASK );  

		PutCharSPI((byte) (invertIt?0x00:0xff), DATA_MASK );  
		// Write a data byte.  
		if(num == 0xffff)
		{
			showIdx=10;
			num=0;
		}
		else
		{
			showIdx=num%10;
			num/=10;
		}
		
		for(j=0;j<3;j++)
		{
			byte v;
			v=nums[showIdx][j];
			if(invertIt)
			{
			    PutCharSPI( v, DATA_MASK );  
			}
			else
			{
			    PutCharSPI((byte) ~v, DATA_MASK );  
			}
		}

		if(num==0)
		{
			break;
		}
	}
	i=Index+(numLen)*4;
	// NOP command is required after writing data before chip select goes high.  
	PutCharSPI( 0xE3, CMD_MASK );  
	return i;
}


// say_my_name() outputs the current generation data 
// I am not a number, I am a free... oh wait. 
// Yeah. 
// Sorry. I _am_ a number. My bad.


// The genOffset is the offset to begin at, in case the rest
// of the data didn't fit on the display...
// XXX this has not been tested.

static word say_my_name(word genOffset)
{
	word m;
	word d;
	word i;
	word idx=BYTES_PER_PAGE+8; // start the actual generation number @ the 2nd line (page 1)
	byte __pmem *toShow;
	word max;
	word showLen;
	
	if(!genOffset)
	{
		
		FillDisplay(0x55,0xaa);
		doDisplay(1,N_ROWS -1);		
		
		if((lfsr&0x1) == (lfsr2&0x1))
		{	
			showLen=90;
			toShow=Banner;
		}
		else
		{
			showLen=64;
			toShow=teamredacted;
		}
		refresh_LFSRs();
		
		max=lfsr2&0x7;
		max+=1;
		if(lfsr&1)
		{
			invertIt=!invertIt;
		}
		for(i=0;i<3;i++)LFSR(lfsr2,0x6000u);
		FillDisplay(0x00,0x00);
		for(m=0;m<=max;m++)
		{
			RedactTeam((128 - showLen)>>1,showLen,toShow,m==max);
			doDisplay(0,8);
		}

		Term_SendStr("Inf.ID:");
	}
	else
	{
		Term_SendStr("Cont:");				
	}

    if(genOffset)
    {
		Term_SendChar('.');
		Term_SendChar('.');
		Term_SendChar('.');
		idx=writeNum(0xffff,idx);
		idx=writeNum(0xffff,idx);
		idx=writeNum(0xffff,idx);
    }
	for(m=GEN_SIZE -( 1+ genOffset); m>curGenIdx;m--)
	{
		d=pmem_read((word*)genFlash+m);
		if(d!=0xffff)
		{
			if(m!= GEN_SIZE - 1)
			{
				Term_SendChar('.');
				idx=writeNum(0xffff,idx);
			}
			Term_SendNum(d);
			idx=writeNum(d,idx);
		}
	}

	d=pmem_read((word*)curFlashed);
	if(d!=0xffff)
	{
		Term_SendChar(':');
		Term_SendNum(d);
		idx+=8;
		if(invertIt)
		{
	    	LoadData(idx+2,1,(byte*)"$"); // '$' == 0x24 == 00100100 == ':'
		}
		else
		{
			LoadData(idx+2,1,(byte*)"\xeb"); // inverted $
		}
    	idx+=4;
		idx=writeNum(d,idx);
	}
	doDisplay(0,N_ROWS);
	return 	(idx >= BYTES_PER_IMAGE );
}

static void do_say_my_name()
{
	word res;
	word gen=0;
	do
	{
		res=say_my_name(gen);
		gen=gen+1;
		if(res)
		{
			Cpu_Delay100US(2500);
		}
	} while(res>0);
	Cpu_Delay100US(5000);
}


// make blinking light/LCD change, update lfsr 
// values and see if we have anything to say.
void sparkleTime()
{
	int i;
	int gotIt=0;
	word theTime;
	
	for(i=8+(int)(((lfsr2^lfsr))&0x7);i>0;i--)
	{
		// 1 chance in about 64 it will report, 
		// but this loop happens between 8 and 15 times so it's quite often really.
		if(!gotIt && (lfsr&0x3f) == 0x12)
		{
			Cpu_DisableInt();
			HardResetDisplay();
			Cpu_EnableInt();
			do_say_my_name();
			Term_CRLF();
			gotIt=1;
		}

		LFSR(lfsr, 0xB400u);
		LFSR(lfsr2,0x6000u);
	}
	if(!gotIt)
	{
		for(i=0;i<8;i++)LFSR(lfsr, 0xB400u); // add 8 new bits
		if((lfsr&0x7) == 0)
		{
			word offset;
			offset=7 + (lfsr &0xf);

			// do banner
			FillDisplay(0x00,0x00);
			
			RedactTeam(0*BYTES_PER_PAGE + offset,90,vpage1,1);
			offset+=(lfsr&0x008)?1:0;
			offset-=(lfsr&0x010)?1:0;
			RedactTeam(1*BYTES_PER_PAGE + offset,90,vpage2,1);
			offset+=(lfsr&0x020)?1:0;
			offset-=(lfsr&0x040)?1:0;
			RedactTeam(2*BYTES_PER_PAGE + offset,90,vpage3,1);
			offset+=(lfsr&0x080)?1:0;
			offset-=(lfsr&0x100)?1:0;
			RedactTeam(3*BYTES_PER_PAGE + offset,90,vpage4,1);
			doDisplay(0,N_ROWS);
		}
		else
		{
			byte v;
			byte w;

			// add some noise
			for(i=0;i<6;i++)LFSR(lfsr2,0x6000u);
			for(i=0;i<5;i++)LFSR(lfsr, 0xB400u);
			v=(byte)(lfsr2&0x1f);
			w=(byte)(lfsr&0x1f);
			if(N_ROWS - v < w)
			{
				w=(byte)(N_ROWS -v);
			}
			// 0x13 == magic value for 'noise'
			FillDisplay(0x13,0x0);
			doDisplay(v,w);
			LFSR(lfsr2,0x6000u);
		}
	}

	// how long to go lower power?

	for(i=0;i<10;i++)LFSR(lfsr2,0x6000u);
	for(i=0;i<11;i++)LFSR(lfsr, 0xB400u);
	theTime = ((lfsr^lfsr2)&0x3fff); // pick a random value
	// set how long I should sleep (in milliseconds)
	wakeupTime(theTime);
	Term_SendChar('w');
	Term_SendChar('=');
	Term_SendNum(theTime);
	Term_CRLF();
	// wait that long
	dc18_sleep();
	LFSR(lfsr, 0xB400u);
	LFSR(lfsr2,0x6000u);
}

void loop() 
{
	// is a button pushed?
	if (intFlag) 
	{
		// runstate 0 == not doing anything, so we can do stuff
		if (systemRunState == 0) 
		{
			int res;
			intFlag = 0;

			// show that something happened.
			FillDisplay((byte) 0xf0,0x00 );
			
			doDisplay(0,N_ROWS /2);
			FillDisplay((byte) 0x05,0xff );
			doDisplay(N_ROWS /2,N_ROWS /2);
			refresh_LFSRs();
			Cpu_Delay100US(8000);

			// how many possible backdoor things can be added? ;/
			// invert display by a button is pushed again here.
			if(intFlag)
			{
				invertIt=!invertIt;
				Term_SendNum(invertIt);
				Term_CRLF();
				FillDisplay((byte) 0x99,0xff );
				doDisplay(N_ROWS /2,N_ROWS /2);
				intFlag=0;
				return;
			}
			// initialize the JTAG state machine, putting the other device into debug mode

			systemRunState = 1;
			intFlag = 0;

			res=doInitTap();

			if(res != 0)
			{
				Term_SendStr("init fail");
				Term_CRLF();
				return;
			}

			// initialize the flash on the other device
			if(once_set_hfmdiv())
			{
				Term_SendStr("hmfdiv fail");
				Term_CRLF();
				systemRunState=13;
				return;
			}
			
			// erase the flash on the other device
			if(once_flash_erase())
			{
				Term_SendStr("erase fail");
				Term_CRLF();
				systemRunState=14;
				return;
			}

			// count here, to prevent dups if it crashes the other device before completing
			incrementCurFlashedCount(); 

			// actually do the infection part.
			if(once_infect_other_badge())
			{
				Term_SendStr("inf fail");
				Term_CRLF();
				systemRunState=15;
				return;
			}
			
			// disconnect, restarting them
			jtag_disconnect();
			
			// smile for the cameras ;]
			Term_SendStr(";]");
			Term_CRLF();
			
			// we are no longer in a weird runstate
			systemRunState=0;
			return; 
		}
	}

	if(systemRunState==0)
	{
		// sparkleTime. 
		// Who doesn't like sparkles?
		sparkleTime();
	}
	else
	{
		static word cnt=0;
		
		Term_SendStr("E=");
		Term_SendNum(systemRunState);
		Term_CRLF();

		if(cnt==0)
		{
			FillDisplay((byte) (~systemRunState ^ 0xc0),0x3f );
			doDisplay(1,N_ROWS -1);
		}

		if(intFlag || (cnt++ > 5) )
		{
			FillDisplay(0x44,0x88);
			doDisplay(1,N_ROWS -1);
			Term_SendStr("rst");
			Term_CRLF();
			systemRunState=0;
			intFlag=0;
			cnt=0;
			return;	
		}
	}
}

/* END jtag_stuff */


#define	FLASH_PROT_VALUE_ADDR	0x1FFA
#define	FLASH_SECHI_VALUE_ADDR	0x1FF8
#define	FLASH_SECLO_VALUE_ADDR	0x1FF7

// apparently FM_PROTB exists on some DSP800E based chips,
// but not on 56F8006. Zero to be consistent.
#define FM_PROTB				(*((volatile word *)0x0000F411))	

// stuff required to make flash writable, 
// based on documentation in various places
static void make_flash_writable()
{
	clrRegBits(FM_CNFG,FM_CNFG_LOCK_MASK);
	setReg16(FM_PROT,0);
	setReg16(FM_PROTB,0); 	 
	setRegBits(FM_USTAT,FM_USTAT_PVIOL_MASK);
	setRegBits(FM_USTAT,FM_USTAT_ACCERR_MASK);
	setRegBits(FM_USTAT,FM_USTAT_BLANK_MASK);

	Mem_SetWordFlash(FLASH_PROT_VALUE_ADDR ,0); 
	Mem_SetWordFlash(FLASH_SECHI_VALUE_ADDR,0);
	Mem_SetWordFlash(FLASH_SECLO_VALUE_ADDR,0);
	
	if(pmem_read((word*)genFlash + GEN_SIZE-1) != __VERSION_NUMBER__)
	{	
		Mem_SetWordFlash(genFlash + GEN_SIZE-1, __VERSION_NUMBER__);
	}
}

// fill screen with either:
//   an alternating pattern of 2 byte values, e.g. 0x55 and 0xaa
// or
//   a single value (when the 2nd value is 0 the magic number 0xf1)
// 
//  The value 0x13 is a magic number which means to use LFSR based noise (play with it.
void FillDisplay(byte z,byte z2)
{
  byte colAddr, page;  
  word i;  
  byte pData;
  word Index=0;

  
  // Segment remapping must be done when loading data.  
  #ifdef REMAP_IMAGE  
    PutCharSPI( 0xA1, CMD_MASK );       // Remap segs.  
  #else  
    PutCharSPI( 0xA0, CMD_MASK );       // Don't remap segs.  
  #endif  

  // Loop through the data to write.  
  for (i = Index; i < Index+BYTES_PER_IMAGE; i++)  
  {    
    if((z2!=0xf1) && (i&1))
    {
    	pData=z2;
    }
    else 
    {
    	pData=z;
    }
    colAddr = (uint8_t)((i % N_COLS) + 2); // Add two to skip unused & frame.  
  
    // Set page and column address as needed.  
    if ((colAddr == 2) || (i == Index) )  
    {  
      page = (uint8_t)((i/N_COLS) + 2); // Add two to skip unused & frame.  

      PutCharSPI( (uint8_t)(COL_MSN | (colAddr / 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(COL_LSN | (colAddr % 16)) , CMD_MASK );  
      PutCharSPI( (uint8_t)(PAGE_ADD | page) , CMD_MASK );  
    }  
  
  	// 

    // Write a data byte.  
	if(pData == 0x13)
	{
		int t;
		// most of the time, do this, otherwise keep the same value...
		if(lfsr2&3)
		{
			// shifT it 8 bits
			for(t=0;t<9;t++)LFSR(lfsr,0xB400u);	
		}
		if(lfsr2&6) // use 2 unused bits
		{
			z^=z2;  // swap z
			z2^=z;  // and z2 
			z^=z2;  // using no extra reg/local variable
		}
		for(t=0;t<4;t++)LFSR(lfsr2,0x6000u); // shift used bits out

		pData=(byte)lfsr;
		
	}
	if(invertIt)
	{
	    PutCharSPI( pData, DATA_MASK );  
	}
	else
	{
	    PutCharSPI((byte) ~pData, DATA_MASK );  
	}
  }  
  
  // NOP command is required after writing data before chip select goes high.  
  PutCharSPI( 0xE3, CMD_MASK );  
}  

void main(void)
{
	word rstat=SIM_RSTAT;
	/* Write your local variable definition here */
	/*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
	PE_low_level_init();
	/*** End of Processor Expert internal initialization.                    ***/

	/* Write your code here */

	// forced it to be an initialized variable, so now i need to 'deinit' it.
	if(lfsr==0xffff)
	{
		lfsr=0;
		lfsr2=0;
	}
	clrWakeup();
	
	Term_SendStr("BOOT");
	switch(rstat)
	{
		
		case SIM_RSTAT_POR_MASK:
		{
			Term_SendStr(".PWR");
			break;
		}

		case SIM_RSTAT_SWR_MASK:
		{
			Term_SendStr("!SWR");
			break;
		}

		case SIM_RSTAT_EXTR_MASK:
		{
			Term_SendStr("!EXTR");
			break;
		}
		default:
		{
			Term_SendStr("RSTAT=");
			showData(rstat,16);
			break;
		}
	}
	Term_CRLF();
	
	// try get some real randomnessfrom the ADC device... 
	// get the raw temperature value, and use that 
	// check thing, but all I want is some random bits.
	Term_SendStr("ADC");
	while( ! (lfsr && lfsr2) )
	{
		if(lfsr == 0)
		{
			AD1_Measure(1);
			AD1_GetValue16(&lfsr);
		}
		Cpu_Delay100US(3000);
		if(lfsr2 == 0)
		{
			AD1_Measure(1);
			AD1_GetValue16(&lfsr2);
		}
		if(lfsr2 == lfsr)
		{
			lfsr2=0; 
		}
		Term_SendChar('.');
	}
	Term_SendStr("ok.");
	Term_CRLF();
	
	Cpu_DisableInt();
	HardResetDisplay();
	Cpu_EnableInt();
	

#if 0 // mask with 3 for badge, 0x3f for demoboard - for LEDs)
	clrReg16Bits(GPIO_A_DR,  0x3fU);
	setReg16Bits(GPIO_A_DDR, 0x3fU); // GPIO_A[0-5].direction = output;
	clrReg16Bits(GPIO_A_PER, 0x3fU); // GPIO_A[0-5].enable_peripheral = no... so GPIO	
#else
	clrReg16Bits(GPIO_A_DR,  0x03U); // GPIO_A[0-1].data = register value
	setReg16Bits(GPIO_A_DDR, 0x03U); // GPIO_A[0-1].direction = output;
	clrReg16Bits(GPIO_A_PER, 0x03U); // GPIO_A[0-1].enable_peripheral = no... so GPIO	
#endif
	
	// load the current generation value from the function. function gets patched on copy
	initCurGenIdx();
	
	// makes the um... flash memory... writable?
	make_flash_writable();
	
	// if the counter is not initialized, then set it to 0.
	if(pmem_read((word*)curFlashed) == 0xffff)
	{
		incrementCurFlashedCount();
	}

	Cpu_Delay100US(4000);

	// update display...
	do_say_my_name();
	doDisplay(0,N_ROWS);
	Cpu_Delay100US(10000);
	
	clearedBits=1;
	intFlag=0;
	systemRunState = 0 ;
		
	for(;;) 
	{
		// just in case
		clrWakeup();
		// do we need to clear the bits? do it, i guess.
		if((!clearedBits)&&(systemRunState != 0))
		{
			Term_SendStr("clrBits");
			Term_CRLF();
			clearBits();
		}
		// main loop -- this code was originally written for arduino, see?
		loop();
	}
}

/* END self_JTAG */
/*
** ###################################################################
**
**     This file was created by Processor Expert 3.00 [04.35]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/
 // Yeah, but it was edited REALLY REALLY REALLY hard by Team Redacted.
 // Mostly by the boys though.
 // . 
